###############################################################################################
# R Code for the analysis of PMI and gene expression for the GTEx v6 dataset
###############################################################################################
# load required libraries
library(MASS)
library(preprocessCore)
library(gplots)
library(RColorBrewer)
library("DOSE")
library("clusterProfiler")
library(KEGG.db)


###########################################################
# Tissue color mapping
###########################################################
tissue.colors.hash = read.table("/home/pferreira/Projects/GTEx2/ischemicTimeStudy/Colors/Tissue.Colors.HASH.V3.txt", h=T,sep="\t")
rownames(tissue.colors.hash) = tissue.colors.hash$tissue
tissue.colors = as.character(tissue.colors.hash$color)

plotTissueColorLegend <- function(){
    # legend for tissue colors
    pdf("legend.Color.tissue.pdf", w=12,h=12)
        par(oma=c(12,0,0,0))
        barplot(rep(1,nrow(tissue.colors.hash)),col=as.character(tissue.colors.hash$color),names=as.character(tissue.colors.hash$tissue),las=2,cex.names=18)
    dev.off()
}

# Expression and annotation datasets belong to the GTEx project; access the GTEx portal or dbgap for appropriate access
#################################################################################
# EXPRESSION DATA
#################################################################################
# read Gene and Transcript expression reads and RPKM values
gene.rpkm <- read.table(gzfile("/home/pferreira/Projects/GTEx2/rna-seq/GTEx_Analysis_2015-01-12_RNA-seq_RNA-SeQCv1.1.8_gene_rpkm.gct.gz"),h=T,sep="\t",skip=2)
rownames(gene.rpkm) = gene.rpkm$Name
# sel.samples.ids corresponds to the 8555 from the midway freeze;
sel.samples.ids = gene.rpkm$Name
sel.samples.ids = chartr(".","-",sel.samples.ids)
#mappings between ensembl ids and gene names
gene.names=gene.rpkm[,c(1,2)]
# import read counts
gene.readcnt = read.table(gzfile("../rna-seq/GTEx_Analysis_2015-01-12_RNA-seq_RNA-SeQCv1.1.8_gene_reads.gct.gz"), header=T, skip=2)
exon.readcnt = read.table(gzfile("../rna-seq/GTEx_Data_20150112_RNAseq_RNASeQCv1.1.8_exon_reads.txt.gz"), header=T)

#################################################################################
# SAMPLE AND SUBJECT ANNOTATION DATA
#################################################################################
# SEQ QC statistics from the RNA-SEQC pipeline
seqqc = read.delim("/home/pferreira/Projects/GTEx2/rna-seq/GTEx_Analysis_2015-01-12_RNA-seq_RNA-SeQCv1.1.8_metrics.tsv",h=T)
# read annotation from the samples
sample_attribs <- read.delim("/home/pferreira/Projects/GTEx2/sample_annotations/GTEx_Analysis_2015-01-12_Annotations_SampleAttributesDS.txt")
subj_sample <- read.delim("sample_annotations/GTEx_Analysis_2015-01-12_Annotations_SubjectSampleMappingDS.txt")
subj_excluded <- read.delim("sample_annotations/excludedSubjects.txt")
subj_pheno <- read.delim("/home/pferreira/Projects/GTEx2/sample_annotations/GTEx_Analysis_2015-01-12_Annotations_SubjectPhenotypesDS.txt")
#merge sample and subject information
sample_attribs_subj = merge(sample_attribs, subj_sample, by.x="SAMPID",by.y="SAMPID")

# RELEVANT ANNOTATION FIELDS
# SAMPID == sample id, SMSTYP == T or N; SMTS == Tissue Type, SMTSD == Details of Tissue Type
# SMTSISCH == Total Ischemic time for a sample
# select a few relevant attributes to have a more manageable table
sel.sample.attribs = c("SAMPID", "SUBJID","SMSTYP","SMTS","SMTSD", "SMTSISCH")
# select samples from the midway data freeze, i.e. those in the expression table
sel.sample_attribs_subj = sample_attribs_subj[sel.sample_attribs_subj$SAMPID %in% sel.samples.ids,]
# add column with names separated by dots and hyphen
sel.sample_attribs_subj = cbind(chartr("-",".",sel.sample_attribs_subj$SAMPID),sel.sample_attribs_subj)
names(sel.sample_attribs_subj)[1] = "SAMPIDDOT"

# add phenotype information of the subject to the table of sample information
sel.sample_attribs_subj_pheno = merge(sel.sample_attribs_subj,subj_pheno,by.x="SUBJID",by.y="SUBJID")
# selected sample variables plus selected covariates for ischemic time
# sel.covars have been selected in the section ANALYSIS OF CO-VARIATES
all.vars = c("SAMPIDDOT",sel.sample.attribs,sel.covars, "DTHHRDY", "TRDNISCH", "DTHVNT")

# filter by the list of samples in the analysis freeze (8555) and the selected variates
sel.sample.subj.covars = sel.sample_attribs_subj_pheno[sel.sample_attribs_subj_pheno$SAMPID %in% sel.samples.ids, colnames(sel.sample_attribs_subj_pheno) %in% all.vars]
rownames(sel.sample.subj.covars)= sel.sample.subj.covars$SAMPIDDOT

#################################################################################
# SELECT TISSUES BY THE RESPECTIVE NUMBER OF SAMPLES
#################################################################################
#select tissues with more than 20 samples
tt = table(sel.sample_attribs_subj$SMTSD)
ttn = names(which(tt[order(tt)]>20))
# Brain samples are either prepared with Tissue:PAXgene Preserved or Tissue:Fresh Frozen Tissue
# the latter does not have ischemic time (1301 samples); the PAX has ischemic time (Cerebellum 161 and Cortex 147)
# remove all brain samples
ttn = grep("Brain",ttn,value=T,invert=T)
# add brain samples with ischemic time
brain.subtissues.withischemictime = as.character(unique(sample_attribs_subj[sample_attribs_subj$SMMTRLTP=="Tissue:PAXgene Preserved" & sample_attribs_subj$SMTS=="Brain",c("SMTSD")]))
ttn=c(ttn,brain.subtissues.withischemictime)
# remove cell lines
ttn=grep("Cells",ttn,value=T,invert=T)
sel.tissues.names = ttn
# number of samples per selected tissue
tt=table(sample_attribs_subj_allpheno[sample_attribs_subj_allpheno$SAMPID %in% sel.samples.ids & sample_attribs_subj_allpheno$SMTSD %in% sel.tissues.names,c("SMTSD")])
# list of samples within the selected tissues
sel.samples.sel.tissues.ids = sample_attribs_subj_allpheno[sample_attribs_subj_allpheno$SAMPID %in% sel.samples.ids & sample_attribs_subj_allpheno$SMTSD %in% sel.tissues.names,c("SAMPID")]
sel.samples.sel.tissues.iddots = chartr("-",".",sel.samples.sel.tissues.ids)

#################################################################################
# ANALYSIS OF CO-VARIATES
#################################################################################
# import annotations
subj.metadata=read.delim("../sample_annotations/../sample_annotations/GTEx_Analysis_2015-01-12_Annotations_SubjectPhenotypesDD.csv",sep=",",h=T)
sample.metadata=read.delim("../sample_annotations/../sample_annotations/",sep=",",h=T)
covars.metadata=rbind(subj.metadata[,c("VARNAME","VARDESC")],sample.metadata[,c("VARNAME","VARDESC")])
sample_attribs_subj_allpheno = merge(sample_attribs_subj,subj_pheno,by.x="SUBJID",by.y="SUBJID")
sel.sample_attribs_subj = cbind(chartr("-",".",sample_attribs_subj_allpheno$SAMPID),ssample_attribs_subj_allpheno)

# covariates are divided in several groups
# Sample (SM), Demographic (no specific prefix), Medical History (MH), Tissue Recovery (TR), Serology results (LB)
# we select covariates of interest in groups SM, Demographic and MH for further analysis
covars.demographic = as.character(subj.metadata[subj.metadata$DOCFILE=="Demography",c(1)])
covars.sample = as.character(sample.metadata[grep("^SM",sample.metadata$VARNAME),c(1)])
covars.medicalhistory = as.character(subj.metadata[grep("^MH",subj.metadata$VARNAME),c(1)])
covars.ofinterest = c(covars.demographic, covars.medicalhistory, covars.sample)

# perform Pearson correlation of these variables with ischemic time
plotCovariateCorrelation <- function()
    # sel.samples.ids corresponds to the 8555 from the midway freeze; extracted from the RPKM matrix; see section "EXPRESSION DATA"
    # new version of the correlation with covariates, correlation with TRISCHD and TRDNISCH
    # done at individual level; select unique individuals for the 
    sel.subj.ids = unique(sample_attribs_subj_allpheno[sample_attribs_subj_allpheno$SAMPID %in% sel.samples.ids,c("SUBJID")])
    cov.corr <- matrix(nrow = length(names(subj_pheno)), ncol = 4, 0);
    covar.mat = subj_pheno[subj_pheno$SUBJID %in% sel.subj.ids,]
    covar.mat = covar.mat[covar.mat$TRISCHD>0,]
    ischemic.time = covar.mat[,"TRISCHD"]

    c = 1
    for (i in seq(1,ncol(covar.mat))){
        covar.name = as.character(names(covar.mat)[i])
        ul = length(unique(covar.mat[,covar.name]))
        isn = is.numeric(covar.mat[,covar.name])
        contains.unknown = length(intersect(c(97,98,99), unique(covar.mat[,covar.name]))) > 0
    
        if((ul>=2 && ul <= 15) || (isn == TRUE)){
            if(covar.name != "TRISCHD"){
                covar2.vals = as.numeric(covar.mat[,covar.name])
                # 97, 98, 99 correspond to unknown values
                contains.unknown = length(intersect(c(97,98,99), unique(covar.mat[,covar.name]))) > 0
                ischvar = cbind(ischemic.time, covar2.vals)
                colnames(ischvar) = c("TRISCHD", covar.name)
                # a categorical variable represented as numbers with unknown/unreported values
                # typically contains less than 10 unique values (4 or 5) and 98 and 99
                if((isn==TRUE) & (ul < 10 ) & (contains.unknown==TRUE)){
                    ischvar = ischvar[ischvar[,covar.name] < 90,]
                }
                fit = lm(ischvar[,"TRISCHD"] ~ ischvar[,covar.name])
                # adjusted RSquare
                adj.rsq = summary(fit)$adj.r.squared
                ct = cor.test(ischvar[,1], ischvar[,2],use="na.or.complete");
                pval = as.numeric(as.vector(ct[3]$p.value));
                
                cat(paste(covar.name, ct$estimate[[1]], sep="\t"),sep="\n")
                cov.corr[c,1] = covar.name
                cov.corr[c,2] = ct$estimate[[1]]
                cov.corr[c,3] = pval
                cov.corr[c,4] = adj.rsq
                c = c + 1
            }
        }
    }
    cov.corr = cov.corr[cov.corr[,1]!="0",]
    colnames(cov.corr) = c("VAR","Pearson.Corr","Pval","AdjR2")
    
    write.table(cov.corr,"correlation.ischemictime.covariates.txt",quote=F,row.names=F)
    cov.corr = read.table("correlation.ischemictime.covariates.txt",h=T)
    # remove covariates with NA correlation 0alue
    cov.corr.nonna = cov.corr[!is.na(cov.corr$Pearson.Corr),] 
    
    #plot the correlation values and color covariates according to the respective groups

    pdf("Correlation.Ischemictime.otherCovariates.v2.pdf",w=18,h=15)
    par(oma=c(6,0,0,0))
    covar.name.list = cov.corr.nonna[order(cov.corr.nonna$Pearson.Corr),c(1)]
    covar.color = rep("gray",length(covar.name.list))
    covar.color[grep("^SM",covar.name.list)] = "green"
    covar.color[grep("^DT",covar.name.list)] = "black"
    covar.color[grep("^LB",covar.name.list)] = "orange"
    covar.color[grep("^MH",covar.name.list)] = "yellow"
    covar.color[grep("^TR",covar.name.list)] = "red"
    barplot(cov.corr.nonna[order(cov.corr.nonna$Pearson.Corr),c(2)],names=covar.name.list,las=2,cex.names=0.8,ylim=c(-0.7,1),col=covar.color,main="Covariate correlation with ischemic time")
    abline(h =seq(-0.5,0.5,by=0.1), col = "lightgray", lty = 3)
    legend("topleft",  c("Sample", "Demographic","Death circumstance","Serology","Medical History","Tissue recovery"), cex=1.5, col=c("green","gray","black","orange","yellow","red"),lty=1, lwd=5,bty="n")
    dev.off()
}


# SAMPLE SEQUENCING FEATURES as described in the annotation files
# SMTSD = Tissue Type, area from which the tissue sample was taken.  This is a parent value to SMTSD.
# SM550NRM = 5' 50-based normalization:  50 (this number is the value for the transcript end length parameter) refers to the definition of how many bases are considered at the end; this value is the ratio between the coverage at the 5' end and the average coverage of the full transcript, averaged over all transcripts; 
# SM350NRM = 3' 50-base normalization: the ratio between the coverage at the 3' end and the average coverage of the full transcript, averaged over all transcripts
# SMNUMGPS	Number of Gaps: Number of regions with ≥5 bases with zero coverage
# SMMNCPB	Mean Coverage Per Base: Coverage is averaged per base across each transcript, and averaged again across all transcripts.
# SMGNSDTC	Genes Detected: Total number of genes with at least 5 exon mapping reads.
# SMTRSCPT	Transcripts Detected: Total number of transcripts with at least 5 exon mapping reads
# SMNTRART = Intragenic Rate: The fraction of reads that map within genes (within introns or exons)
# SMEXNCRT = Exonic Rate: The fraction of reads that map within exons
# SMNTERRT = Intergenic Rate: The fraction of reads that map to the genomic space between genes
# SMNTRNRT = Intronic Rate: The fraction of reads that map within introns
# SMEXPEFF	Expression Profiling Efficiency: Ratio of exon reads to total reads
# SMMAPRT	Mapping Rate: Ratio of total mapped reads to total reads
# SMRRNART	rRNA Rate: Ratio of all reads aligned to rRNA regions to total reads
# ratio = SM550NRM/SM350NRM ratio of 5' and 3' normalized coverage
# SM550NRM = 5' 50-based normalization:  50 (this number is the value for the transcript end length parameter) refers to the definition of how many bases are considered at the end; this value is the ratio between the coverage at the 5' end and the average coverage of the full transcript, averaged over all transcripts; 
# SM350NRM = 3' 50-base normalization: the ratio between the coverage at the 3' end and the average coverage of the full transcript, averaged over all transcripts
# ratio = SM550NRM/SM350NRM ratio of 5' and 3' normalized coverage
sel.tissues.names = sel.tissues.names[order(sel.tissues.names)]

plotSampleAndSequencingFeatures <- function(){
    # create a partial view of the covariates to be plotted
    a = covar.mat[,c("SMTSD","SMTSISCH", "SMRIN" , "SM550NRM", "SM350NRM" ,"SMNTRART","SMEXNCRT", "SMNTERRT", "SMNTRNRT", "SMNUMGPS", "SMMAPRT", "SMUNMPRT", "SMMNCPB","SMEXPEFF","SMRRNART","SMGNSDTC","SMTRSCPT")]
    a = cbind(a,a$SM550NRM/a$SM350NRM)
    names(a)[length(names(a))] = "ratio"
    a = a[!is.na(a$ratio),]
    a = a[a$SMTSD %in% sel.tissues.names,]
    a = droplevels(a)

    # distribution of number of genes per tissue
    pdf("SampleAttrib.NumGenesDetected.pdf", w=16,h=12, useDingbats=FALSE) # SMGNSDTC
        tc = as.character(tissue.colors.hash[sel.tissues.names,c(2)])
        par(oma=c(12,0,0,0))
        boxplot(a$SMGNSDTC ~ a$SMTSD,las=2,at=rank(tapply(a$SMGNSDTC, a$SMTSD,median),ties.method ="first"),col=tc)
    dev.off()
    # Genomic mapping rates
    pdf("SampleAttrib.MappingStats.Intragenic.Exonic.Intergenic.Intronic.pdf",w=20,h=20, useDingbats=FALSE)
        rr = rank(tapply(a$SMNTRART, a$SMTSD,median),ties.method ="first")
        tc = as.character(tissue.colors.hash[sel.tissues.names,c(2)])
        par(mfrow=c(2,2))
        par(oma=c(12,0,0,0))
        boxplot(a$SMNTRART ~ a$SMTSD,las=2,at=rr,col=tc, main="Intragenic Rate",names=NULL)
        boxplot(a$SMEXNCRT ~ a$SMTSD,las=2,at=rr,col=tc, main = "Exonic Rate",names=NULL)
        boxplot(a$SMNTERRT ~ a$SMTSD,las=2,at=rr,col=tc, main = "Intergenic Rate")
        boxplot(a$SMNTRNRT ~ a$SMTSD,las=2,at=rr,col=tc, main = "Intronic Rate")
    dev.off()
    # different variables per tissue
    pdf("SampleAttrib.MappingStats.ExpEfficiency.mapRate.rRNArate.5P3Prate.pdf",w=20,h=20, useDingbats=FALSE)
        rr = rank(tapply(a$SMEXPEFF, a$SMTSD,median),ties.method ="first")
        tc = as.character(tissue.colors.hash[sel.tissues.names,c(2)])
        par(mfrow=c(2,2))
        par(oma=c(12,0,0,0))
        boxplot(a$SMEXPEFF ~ a$SMTSD,las=2,at=rr,col=tc, main="Expression Profiling Efficiency",names=NULL)
        boxplot(a$SMMAPRT ~ a$SMTSD,las=2,at=rr,col=tc, main = "Mapping Rate",names=NULL)
        boxplot(a$SMRRNART ~ a$SMTSD,las=2,at=rr,col=tc, main = "rRNA Rate")
        boxplot(a$ratio ~ a$SMTSD,las=2,at=rr,col=tc, main = "5' / 3' Normalized Coverage ratio")
    dev.off()
    # distribution of 5', 3'and 5'/3'rates per tissue
    pdf("SampleAttrib.5Prate.3Prate.5P3Prate.pdf",w=20,h=30,pointsize=26, useDingbats=FALSE)
        rr = rank(tapply(a$ratio, a$SMTSD,median),ties.method ="first")
        tc = as.character(tissue.colors.hash[sel.tissues.names,c(2)])
        par(mfrow=c(3,1))
        par(oma=c(12,0,0,0))
        boxplot(a$SM550NRM ~ a$SMTSD,las=2,at=rr,col=tc, main="5' - 50bp Normalized Coverage ratio",names=NULL)
        boxplot(a$SM350NRM ~ a$SMTSD,las=2,at=rr,col=tc, main = "3' - 50bp Normalized Coverage ratio",names=NULL)
        boxplot(a$ratio ~ a$SMTSD,las=2,at=rr,col=tc, main = "5' / 3' - 50bp Normalized Coverage ratio")
    dev.off()
    # PMI (ischemic time) and 5', 3'and 5'/3'rate relation    
    pdf("SampleAttrib.RIN.ISCHEMIC.3Pratio.5P3Pratio.pdf",w=16,h=16,pointsize=20, useDingbats=FALSE)
        par(mfrow=c(2,2))
        plot(a$SMTSISCH,a$SM350NRM,col="blue",xlab="Ischemic time",ylab="3' - 50bp Normalized Coverage ratio", main="R = 0.23")
        plot(a$SMRIN,a$SM350NRM,col="blue",xlab="RIN",ylab="3' - 50bp Normalized Coverage ratio", main="R = -0.46",xlim=c(6,10))
        plot(a$SMTSISCH,a$ratio,col="darkolivegreen",xlab="Ischemic time",ylab="5' / 3' - 50bp Normalized Coverage ratio", main="R = -0.2")
        plot(a$SMRIN,a$ratio,col="darkolivegreen",xlab="RIN",ylab="5' / 3' - 50bp Normalized Coverage ratio", main="R = 0.4",xlim=c(6,10))
    dev.off()

    # Genomic Rates
    # SMNTRART = Intragenic Rate: The fraction of reads that map within genes (within introns or exons)
    # SMEXNCRT = Exonic Rate: The fraction of reads that map within exons
    # SMNTERRT = Intergenic Rate: The fraction of reads that map to the genomic space between genes
    # SMNTRNRT = Intronic Rate: The fraction of reads that map within introns
    #cor(covar.mat$SMTSISCH, covar.mat$SMNTRART,use="na.or.complete")
    #[1] -0.02586291
    #> cor(covar.mat$SMTSISCH, covar.mat$SMEXNCRT,use="na.or.complete")
    #[1] -0.199464
    #> cor(covar.mat$SMTSISCH, covar.mat$SMNTERRT,use="na.or.complete")
    #[1] 0.02676741
    #> cor(covar.mat$SMTSISCH, covar.mat$SMNTRNRT,use="na.or.complete")
    #[1] 0.223198
    pdf("GenomicReadRate.PMI.pdf",w=16,h=16,pointsize=20, useDingbats=FALSE)
    #png("GenomicReadRate.PMI.png",w=2000,h=2000,pointsize=20)
    par(mfrow=c(2,2))
        plot(covar.mat$SMTSISCH, covar.mat$SMNTRART, xlab="PMI", ylab="Genomic Read Rate",main="Intragenic rate vs PMI: r=-0.025", col="deepskyblue")
        plot(covar.mat$SMTSISCH, covar.mat$SMEXNCRT, xlab="PMI", ylab="Genomic Read Rate",main="Exonic rate vs PMI: r=-0.2", col="firebrick1")
        plot(covar.mat$SMTSISCH, covar.mat$SMNTERRT, xlab="PMI", ylab="Genomic Read Rate",main="Intergenic rate vs PMI: r=-0.026", col="gold1")
        plot(covar.mat$SMTSISCH, covar.mat$SMNTRNRT, xlab="PMI", ylab="Genomic Read Rate",main="Intronic rate vs PMI: r=0.22", col="green1")
    dev.off()
}


# calculate a table with number of mapped reads (mean and stdev) and mapping rate
reportMappingStatsByTissue <- function(){
    # SMMPPD	Mapped: Total number of reads aligned/mapped
    # SMMAPRT	Mapping Rate: Ratio of total mapped reads to total reads
    st1 = aggregate(covar.mat[,c("SMTSD","SMMPPD")],list(covar.mat$SMTSD), mean)
    st2 = aggregate(covar.mat[,c("SMTSD","SMMPPD")],list(covar.mat$SMTSD), sd)
    st3 = aggregate(covar.mat[,c("SMTSD","SMMAPRT")],list(covar.mat$SMTSD), mean)
    st4 = aggregate(covar.mat[,c("SMTSD","SMMAPRT")],list(covar.mat$SMTSD), sd)
    mapping.stats = cbind(st1, st2, st3, st4)
    mapping.stats = mapping.stats[,c(1,3,6,9,12)] 
    names(mapping.stats) = c("tissue", "Mean.MappedReads","Sd.MappedReads","Mean.MapRate","Sd.MapRate")
    sel.tissues.names = mapping.stats[sel.tissues.names,]
    write.table(round(mapping.stats[,-c(1)],2),"mappingstats.txt",quote=F,sep="\t")
}

# calculate the correlation between ischemic time and 3p and 5'/3'
# usage: wrapperCorrelationIschemic3pNorm(sel.tissues.names)
wrapperCorrelationIschemic3pNorm <- function(sel.tissues.names){
    corr.ischemic.NormCov = matrix(0,nrow=length(sel.tissues.names), ncol=4)
    i = 1;
    for (t in sel.tissues.names){
        b = a[a$SMTSD == t,c("SMTSISCH","SM550NRM", "SM350NRM", "ratio")]
        r.5p = cor(b$SMTSISCH, b$SM550NRM, method="pearson", use = "pairwise.complete.obs");
        r.3p = cor(b$SMTSISCH, b$SM350NRM, method="pearson", use = "pairwise.complete.obs");
        r.ratio = cor(b$SMTSISCH, b$ratio, method="pearson", use = "pairwise.complete.obs");
        corr.ischemic.NormCov[i, 1] = t;
        corr.ischemic.NormCov[i, 2] = r.5p;
        corr.ischemic.NormCov[i, 3] = r.3p;
        corr.ischemic.NormCov[i, 4] = r.ratio;
        cat(paste(t,r.ratio, r.5p, r.3p,sep="\t"), sep="\n")
        i = i + 1
    }
    write.table(corr.ischemic.NormCov, "correlation.byTissue.ischemicTime.5p3p_NormCov.txt", quote=F, sep="\t",row.names=F)
}

# plot the correlation of the ischemic time (PMI) with the ratio of the 5' / 3' - 50bp Normalized Coverage ratio per tissue
plotCorrelationIschemic3pNorm <- function(sel.tissues.names){
    wrapperCorrelationIschemic3pNorm(sel.tissues.names)
    corr.ischemic.NormCov = read.table("correlation.byTissue.ischemicTime.5p3p_NormCov.txt", sep="\t",h=T)
    pdf("SampleAtrrib.barplot.corr.ischemicTime.NormCoverageRatio5p3p.pdf", w=16,h=16, useDingbats=FALSE)
        par(oma=c(12,0,0,0))
        tc = as.character(tissue.colors.hash[sel.tissues.names,c(2)])
        barplot(corr.ischemic.NormCov$V4[order(corr.ischemic.NormCov$V4)], names.arg = corr.ischemic.NormCov$V1[order(corr.ischemic.NormCov$V4)],las=2,ylim=c(-0.7,0.2),col=tc[order(corr.ischemic.NormCov$V4)],xpd=F)
    dev.off()
}

###########################################################
# Plot distribution of ischemic time by subtissue
###########################################################
plotPMIbyTissue <- function(){
    subtisse.ischemictime = sel.sample.subj.covars[sel.sample.subj.covars$SMTSD %in% sel.tissues.names,c("SMTSD","SMTSISCH")]
    subtisse.ischemictime=droplevels(subtisse.ischemictime)
    subtisse.ischemictime=subtisse.ischemictime[!is.na(subtisse.ischemictime$SMTSISCH),]

    pdf("boxplot.distribution.ischemicTime.byTissue.pdf", w=24,h=18, pointsize=16, useDingbats=FALSE)
        par(oma=c(12,0,0,0))
        ## boxplot assumes that the order of the elements to be plotted is alphabetical order
        ## so we need to first order tissue by their alphabetical order and pass the respective color vector
        ## boxplot then handles the new order provide in at with the rank function, so no need to reorder according to rank order
        tc = as.character(tissue.colors.hash[sel.tissues.names,c(2)])
        boxplot(subtisse.ischemictime$SMTSISCH ~subtisse.ischemictime$SMTSD,col=tc,las=2,at=rank(tapply(subtisse.ischemictime$SMTSISCH, subtisse.ischemictime$SMTSD,median),ties.method ="first"))
    dev.off()
}

###########################################################
# Plot number of samples per tissue
###########################################################
plotSampleSizebyTissue <- function(){
    subtissue.count = table(sel.sample.subj.covars$SMTSD)
    subtissue.count=subtissue.count[sel.tissues.names]
    subtissue.count=subtissue.count[order(subtissue.count)]
    tc = as.character(tissue.colors.hash[names(subtissue.count),c(2)])

    pdf("barplot.selected.tissues.size.pdf", w=24,h=18, pointsize=16, useDingbats=FALSE)
        par(oma=c(12,0,0,0))
        barplot(subtissue.count,names=names(subtissue.count),las=2,col=tc,main="Number of samples per tissue")
    dev.off()
}

# get the sample ids of a given tissue
getTissueSampleIds <-function(tissue.name){
    return (sel.sample.subj.covars[sel.sample.subj.covars$SMTSD==tissue.name,c("SAMPIDDOT")])
}

######################################################################################################################
# CORRELATION OF GENE EXPRESSION WITH ISCHEMIC TIME ANALYSIS
######################################################################################################################

###########################################################
# Compute Correlation based on Ischemic Time of Sample
# 1) extract the effect of other variables in gene expression
# 2) perform regression and extract residuals;
# 3) use residuals as expression phenotype and compute correlation
###########################################################
#usage: runCorrIschemicSampleRegressionCovariates(sel.sample.subj.covars, selected.samples, "tt.txt", 0.4, covars.list)
runCorrIschemicSampleRegressionCovariates <- function(sample.matrix, selected.samples, output_table_file, min.output.corrvalue, covars.list, randomize){
    library(preprocessCore)
    # count genes with corr >= min.output.corrvalue
    num_genes = 0
    # Prepare COVARIATE DATA: filter the covariate data to add to the regression model
    # filter for the samples of the tissue under analysis
    sel.sample.matrix = sample.matrix[sample.matrix$SAMPIDDOT %in% selected.samples,]
    # get the values for the ischemic time
    ischemic.time = as.numeric(sel.sample.matrix[sel.sample.matrix$SAMPIDDOT %in% selected.samples,c("SMTSISCH")])
    # remove columns with variates not to be included in the regression and Non-numerical variates that need to be converted before
    sel.sample.matrix.filtered = sel.sample.matrix[, covars.list]
    # one case with NA value in MHCANCERNM is imputed to 0  
    sel.sample.matrix.filtered[is.na(sel.sample.matrix.filtered)] = 0
    
    # Non-numerical covariates:
    #"SMCAT" "SMCENTER" "SMTSTPTREF" "SMNABTCHT" "COHORT" "GENDER"
    # if each of the variates have at least 2 or more values add them to the matrix after conversion
    
    # Indicates the type and color of the sample kit used in sample collection
    nf = length(unique(sel.sample.matrix$SMCAT))
    if(nf>1){sel.sample.matrix.filtered = cbind(sel.sample.matrix.filtered, as.numeric(sel.sample.matrix$SMCAT));names(sel.sample.matrix.filtered)[ncol(sel.sample.matrix.filtered)]="SMCAT";}
    # Indicates the site that enrolled the donor, brain subregions may have more than one site
    nf = length(unique(sel.sample.matrix$SMCENTER))
    if(nf>1){sel.sample.matrix.filtered = cbind(sel.sample.matrix.filtered, as.numeric(sel.sample.matrix$SMCENTER));names(sel.sample.matrix.filtered)[ncol(sel.sample.matrix.filtered)]="SMCENTER";}
    # Indicates whether the sample ischemic time begins at the donor's actual death, presumed death, or when the cross clamp was applied in the case of surgical donors
    nf = length(unique(sel.sample.matrix$SMTSTPTREF))
    if(nf>1){sel.sample.matrix.filtered = cbind(sel.sample.matrix.filtered, as.numeric(sel.sample.matrix$SMTSTPTREF));names(sel.sample.matrix.filtered)[ncol(sel.sample.matrix.filtered)]="SMTSTPTREF";}
    # The process by which DNA/RNA was isolated
    nf = length(unique(sel.sample.matrix$SMNABTCHT))
    if(nf>1){sel.sample.matrix.filtered = cbind(sel.sample.matrix.filtered, as.numeric(sel.sample.matrix$SMNABTCHT));names(sel.sample.matrix.filtered)[ncol(sel.sample.matrix.filtered)]="SMNABTCHT";}
    #The study subgroup that the participant belongs to. This indicates whether the participant was a postmortem donor, an organ donor, or a surgical donor.
    nf = length(unique(sel.sample.matrix$COHORT))
    if(nf>1){sel.sample.matrix.filtered = cbind(sel.sample.matrix.filtered, as.numeric(sel.sample.matrix$COHORT));names(sel.sample.matrix.filtered)[ncol(sel.sample.matrix.filtered)]="COHORT";}
    # gender
    nf = length(unique(sel.sample.matrix$GENDER))
    if(nf>1){sel.sample.matrix.filtered = cbind(sel.sample.matrix.filtered, as.numeric(sel.sample.matrix$GENDER));names(sel.sample.matrix.filtered)[ncol(sel.sample.matrix.filtered)]="GENDER";}
    rm(sel.sample.matrix)
    
    # PREPARE GENE EXPRESSION DATA
    # selects the gene expression for the input samples; first do a log2 transformation
    # then do a quantile normalization
    rpkm.tissue = gene.rpkm[,colnames(gene.rpkm) %in% selected.samples];
    rpkm.tissue = log2(0.5+rpkm.tissue);
    rpkm = as.data.frame(normalize.quantiles(as.matrix(rpkm.tissue)));
    names(rpkm) = names(rpkm.tissue);
    rownames(rpkm) = rownames(rpkm.tissue);
    rm(rpkm.tissue)
    
    # compute the correlations
    pval.mat.res <- matrix(nrow=nrow(rpkm),ncol=6);
    for (i in 1:nrow(rpkm)){
        gname = as.character(rownames(rpkm[i,]))
        gexp = as.numeric(rpkm[i,]);
        gexpraw = 2^gexp -0.5
        
        if(!missing(randomize)){
            cat("Randomize gene expression",sep="\n")
            gexp = sample(gexp)
        }
        
        # select genes with minimal expression
        if(mean(gexpraw)>0.5){
            reg = lm(gexp ~ as.matrix(sel.sample.matrix.filtered))  # regress model
            gexp.resid =  residuals(reg)                            # get the residuals
            
            # get correlation values
            cc.pearson = cor.test(gexp.resid,ischemic.time,method="pearson",use="na.or.complete")
            cc.spearman = cor.test(gexp.resid,ischemic.time,method="spearman")
            cc.pearson.cor = as.numeric(cc.pearson$estimate)
            cc.pearson.pval = as.numeric(cc.pearson$p.value)
            cc.spearman.cor = as.numeric(cc.spearman$estimate)
            cc.spearman.pval = as.numeric(cc.spearman$p.value)
            
            if(abs(cc.pearson.cor)>min.output.corrvalue & abs(cc.spearman.cor)>min.output.corrvalue){
                cat(paste(i,gname,cc.pearson.cor,cc.pearson.pval,cc.spearman.cor,cc.spearman.pval,sep=" "),sep="\n")
                num_genes = num_genes + 1
            }
            pval.mat.res[i,1]=i
            pval.mat.res[i,2]=as.character(gname)
            pval.mat.res[i,3]=cc.pearson.cor
            pval.mat.res[i,4]=cc.pearson.pval
            pval.mat.res[i,5]=cc.spearman.cor
            pval.mat.res[i,6]=cc.spearman.pval
        }
    }
    names(pval.mat.res) = c("index","genename","pearson.cor","pearson.pval","spearman.cor","spearman.pval")
    # output to text file
    write.table(pval.mat.res[!is.na(pval.mat.res[,3]),],output_table_file,sep="\t")
    cat(paste("Num genes with corr >=", min.output.corrvalue, ": ", num_genes),sep="\n")
}

##################################################################################
# call regression with expression residuals
# wrapper function for all tissues
##################################################################################
runCorrIschemicSampleRegressionCovariatesAllTissues <- function(sample.matrix, sel.tissues.names, min.out.corr.val, covars.list) {
    sel.tissues.names.nospaces = chartr(" ",".",sel.tissues.names)
    for (i in 1:length(sel.tissues.names)){
        tissue.name = sel.tissues.names[i]
        tissue.name.nospace = sel.tissues.names.nospaces[i]
        output_table_file.name = paste("Regression.Residuals.",sel.tissues.names.nospaces[i],".txt",sep="")
        cat(tissue.name,sep="\n")
        selected.tissue.samples = sample.matrix[sample.matrix$SMTSD==tissue.name,c("SAMPIDDOT")]
        runCorrIschemicSampleRegressionCovariates(sample.matrix, selected.tissue.samples, output_table_file.name, min.out.corr.val, covars.list)
    }
}

# function to run the correlation analysis between gene expression and PMI
runCorrelationGeneExpPMI <- function(){
    # minimum gene output correlation value: set this value accordingly
    min.out.corr.val = 0.1
    # selected covariates + SMCAT","SMCENTER","SMTSTPTREF","SMNABTCHT", "COHORT" that are hard-coded in the function because they are non-numeric and need transformation
    covars.list = c("SMRIN", "MHCANCERNM", "HGHT", "SMATSSCR", "AGE", "ETHNCTY") 
    runCorrIschemicSampleRegressionCovariatesAllTissues(sel.sample.subj.covars, sel.tissues.names, min.out.corr.val, covars.list)
}

###############################################################
# select blood samples with positive ischemic time (post-mortem)
###############################################################
runIschemicTimeForWholeBloodPostMortem <- (){
    tissue.name = "Whole Blood"
    blood.samples.postmortem = sel.sample.subj.covars[sel.sample.subj.covars$SMTSD==tissue.name & sel.sample.subj.covars$SMTSISCH>0,c("SAMPIDDOT")]
    tissue.name =  chartr(" ",".",tissue.name)
    output_table_file.name = paste("Regression.Residuals.",tissue.name,".txt",sep="")
    # run correlation
    runCorrIschemicSampleRegressionCovariates(sel.sample.subj.covars, blood.samples.postmortem, output_table_file.name, min.out.corr.val, covars.list)
}




###############################################################################################
# Plot density distribution of correlations with ischemic time: boxplots or density plots
###############################################################################################
# read to variables the tables with the correlation values
# usage:
tissue.colors = as.character(tissue.colors.hash[sel.tissues.names,c("color")])
plotTissueCorrs(sel.tissues.names, tissue.colors, 70000, "density.correlation.alltissues.pdf", "")
plotTissueCorrs(sel.tissues.names, tissue.colors, 160000, "Regression.Residuals.density.correlation.alltissues.pdf", "Regression.Residuals.")

# plot the density of the distribution of the correlation of gene expression with ischemic time
plotTissueCorrs <- function(sel.tissues.names, sel.tissue.colors, ymax, out.filename, prefix.filename){
    sel.tissues.names.nospaces = chartr(" ",".",sel.tissues.names)
    dmax <- matrix(nrow = length(sel.tissues.names), ncol=1, 0);
    # DENSITY DISTRIBUTION PLOT
    pdf(out.filename, w=15, h=11.25, pointsize=16, useDingbats=FALSE)
        # plot first tissue
        i = 1
        tissue.name = sel.tissues.names[i]
        tissue.name.nospace = sel.tissues.names.nospaces[i]
        output_table_file.name = paste(prefix.filename, sel.tissues.names.nospaces[i],".txt",sep="")
        cat(output_table_file.name, sep="\n")
        # read the table with correlation info
        tissue.corr=read.table(output_table_file.name,h=T)
        colnames(tissue.corr) = c("index","genename","pearson.cor","pearson.pval","spearman.cor","spearman.pval")
        rownames(tissue.corr) = tissue.corr$genename
        corr.list.alltissues <- list()
        corr.list.alltissues[i] = list(tissue.corr$pearson.cor)
        d=density(tissue.corr$pearson.cor)
        
        plot(d$x,length(tissue.corr$pearson.cor)*d$y,type="l",xlim=c(-1,1),ylim=c(0,ymax),lwd=2,col = sel.tissue.colors[i] ,xlab="Pearson correlation with ischemic time",ylab="")
        for (i in 2:length(sel.tissues.names)){
            tissue.name = sel.tissues.names[i]
            tissue.name.nospace = sel.tissues.names.nospaces[i]
            output_table_file.name = paste(prefix.filename, sel.tissues.names.nospaces[i],".txt",sep="")
            cat(output_table_file.name, sep="\n")
            # read the table with correlation info
            tissue.corr=read.table(output_table_file.name,h=T)
            colnames(tissue.corr) = c("index","genename","pearson.cor","pearson.pval","spearman.cor","spearman.pval")
            rownames(tissue.corr) = tissue.corr$genename
            d=density(tissue.corr$pearson.cor)
            lines(d$x,length(tissue.corr$pearson.cor)*d$y,type="l",xlim=c(-1,1),lwd=2, col = sel.tissue.colors[i])
            corr.list.alltissues[i] = list(tissue.corr$pearson.cor)
            dmax[i] = max(length(tissue.corr$pearson.cor)*d$y);
        }
        names(corr.list.alltissues) = paste("A",seq(1,34),sep="")
        legend("topright",sel.tissues.names,col=sel.tissue.colors, cex=1, lwd=2, bty="n")
    dev.off()
    # BOXPLOT
    pdf(paste(prefix.filename, "boxplot.correlation.Regression.Residuals.alltissues.pdf"),w=16,h=14)
        par(oma=c(12,0,0,0))
        # the at parameter allows to rank the variables by median
        # tc = as.character(tissue.colors.hash[order(tissue.colors.hash$tissue),c(2)])
        boxplot(corr.list.alltissues,names=sel.tissues.names,las=2,ylim=c(-1,1),col=sel.tissue.colors, at=rank(sapply(corr.list.alltissues,IQR)))
    dev.off()
}

################################################################################
# STATISTICS ON CORRELATED GENES
################################################################################
# Filter genes according to the correlation values, the adjusted pvalue or the direction of correlation
all.cor.genes = filterSignificantlyCorrelatedGenes(sel.tissues.names, 0.2, 1, "residualsregression/Regression.Residuals.", 0)
pos.cor.genes = filterSignificantlyCorrelatedGenes(sel.tissues.names, 0.2, 1, "residualsregression/Regression.Residuals.", 1)
neg.cor.genes = filterSignificantlyCorrelatedGenes(sel.tissues.names, -0.2, 1, "residualsregression/Regression.Residuals.", -1)
# add biotype
pos.cor.genes = merge(pos.cor.genes, bt, by.x="genename", by.y="id")
neg.cor.genes = merge(neg.cor.genes, bt, by.x="genename", by.y="id")

# for all the tissues calculate the percentage of biotype of genes with correlation with ischemic time
# biotypeDistributionGenesCorrelatedIschemicTime(pos.cor.genes, "percentage.biotype.pos.corr.genes.txt")
# biotypeDistributionGenesCorrelatedIschemicTime(neg.cor.genes, "percentage.biotype.neg.corr.genes.txt")
biotypeDistributionGenesCorrelatedIschemicTime <- function(gene.set, out.filename){
    sel.tissues.names.nospaces = chartr(" ",".",sel.tissues.names)
    bt.distribution = c("Tissue","totalGenes","PercPCoding","PercLincRNA","percProcessedTrx")
    for(tissue.name in sel.tissues.names.nospaces){
        totalnumgenes = as.numeric(nrow(gene.set[gene.set$tissue==tissue.name,]));
        pcod = as.numeric((table(gene.set[gene.set$tissue==tissue.name,c("bt")])/ nrow(gene.set[gene.set$tissue==tissue.name,]) * 100)["protein_coding"]);
        lincRNA = as.numeric((table(gene.set[gene.set$tissue==tissue.name,c("bt")])/ nrow(gene.set[gene.set$tissue==tissue.name,]) * 100)["lincRNA"]);
        processed_trans = as.numeric((table(gene.set[gene.set$tissue==tissue.name,c("bt")])/ nrow(gene.set[gene.set$tissue==tissue.name,]) * 100)["processed_transcript"]);
        cat(paste(tissue.name, pcod, lincRNA, processed_trans, sep="\t"),sep="\n");
        bt.distribution = rbind(bt.distribution, c(tissue.name, totalnumgenes, pcod, lincRNA, processed_trans))
    }
    write.table(bt.distribution, out.filename, sep="\t",quote=F,col.names=F);
}

# load the files with correlation with regression residuals previously calculated and divided in the different tissues
# output the correlation value for each gene (separate tissues by different files)
filterSignificantlyCorrelatedGenes <- function(sel.tissues.names, min_abs_corr, max_padj, prefix.filename, corr.type){
    sel.tissues.names.nospaces = chartr(" ",".",sel.tissues.names)
    all.sig.genes = c()
    sig.genes.stats = matrix(nrow=length(sel.tissues.names), ncol=4,0)
    
    for (i in 1:length(sel.tissues.names)){
        tissue.name = sel.tissues.names[i];
        tissue.name.nospace = sel.tissues.names.nospaces[i];
        output_table_file.name = paste(prefix.filename, sel.tissues.names.nospaces[i],".txt",sep="")
        # read the table with correlation info
        if(file.exists(output_table_file.name)){
            tissue.corr=read.table(output_table_file.name,h=T);
            colnames(tissue.corr) = c("index","genename","pearson.cor","pearson.pval","spearman.cor","spearman.pval");
            rownames(tissue.corr) = tissue.corr$genename;
            tissue.corr=cbind(tissue.corr,p.adjust(tissue.corr$pearson.pval,method="BH"));
            names(tissue.corr)[7]="padj";
            tissue.corr=merge(tissue.corr,gene2name,by.x="genename",by.y="Name")
            # positive correlations
            if(corr.type == 1){
                sig.genes = tissue.corr[(tissue.corr$pearson.cor)> min_abs_corr,c("Description","genename","pearson.cor","padj")]
            }
            #negative correlations
            if(corr.type == -1){
                    sig.genes = tissue.corr[(tissue.corr$pearson.cor) < min_abs_corr,c("Description","genename","pearson.cor","padj")]
            }
            # all correlations
            if(corr.type == 0){
                    sig.genes = tissue.corr[(abs(tissue.corr$pearson.cor) > min_abs_corr) ,c("Description","genename","pearson.cor","padj")]
                    sig.genes.2 = tissue.corr[(abs(tissue.corr$pearson.cor) > (min_abs_corr + 0.1)) ,c("Description","genename","pearson.cor","padj")]
                    # consider these as the genes to be counted
                    # comment the previous 2 instructions
                    #sig.genes = tissue.corr[(abs(tissue.corr$pearson.cor) > min_abs_corr) & (tissue.corr$padj < max_padj) ,c("Description","genename","pearson.cor","padj")]
                    #sig.genes.2 = tissue.corr[(abs(tissue.corr$pearson.cor) > (min_abs_corr +0.1)) & (tissue.corr$padj < max_padj),c("Description","genename","pearson.cor","padj")]
            }
            # select b pvalue instead of correlation
            if(corr.type == 3){
                sig.genes = tissue.corr[(tissue.corr$padj < max_padj) ,c("Description","genename","pearson.cor","padj")]
            }
            # add here the gene list to be counted
            sig.genes = cbind(sig.genes, rep(tissue.name.nospace,nrow(sig.genes)))
            names(sig.genes)[5] = "tissue";
            
            ratio.sig.genes = as.numeric(sprintf("%.4f", (nrow(sig.genes)/nrow(tissue.corr))))*100;
            #ratio.sig.genes.2 = as.numeric(sprintf("%.4f", (nrow(sig.genes.2)/nrow(tissue.corr))))*100;
            min.corr = min(tissue.corr$pearson.cor)
            max.corr = max(tissue.corr$pearson.cor)
            median.corr = median(tissue.corr$pearson.cor)
            iqr = IQR(tissue.corr$pearson.cor)
            cat(paste(tissue.name, nrow(tissue.corr), nrow(sig.genes), ratio.sig.genes, min.corr, median.corr, max.corr, iqr, sep=","),sep="\n")
            sig.genes.stats[i,1] = tissue.name
            sig.genes.stats[i,2] = nrow(tissue.corr)
            sig.genes.stats[i,3] = nrow(sig.genes)
            sig.genes.stats[i,4] = ratio.sig.genes
            #write.table(sig.genes, output_table_file.name, quote=F, row.names=F,sep="\t")
            all.sig.genes = rbind(all.sig.genes, sig.genes)
        }
    }
    res.sig.genes = list(all.sig.genes = all.sig.genes, tissue.stats = sig.genes.stats)
    cat(paste("GENES FILTERED: ", nrow(all.sig.genes)), sep="\n")
    return (res.sig.genes)
}


# for genes only with FDR< 1%
setwd("/home/pferreira/Projects/GTEx2/ischemicTimeStudy/LinearRegression/residualsregression/")
all.sig.FDR0.01.genes = filterSignificantlyCorrelatedGenes(sel.tissues.names, 0.2, 0.01, "/home/pferreira/Projects/GTEx2/ischemicTimeStudy/LinearRegression/residualsregression/Regression.Residuals.",3) 
write.table(all.sig.FDR0.01.genes,"Regression.Residuals.all.sig.FDR0.01.genes.txt",sep="\t")

# type can be positive correlations == 1; negative correlations == -1; all correlation == 0
filterTopCorrelatedGenes <- function(sel.tissues.names, n_ranked_genes, prefix.filename, corr.type){
    sel.tissues.names.nospaces = chartr(" ",".",sel.tissues.names)
    all.sig.genes = c()
    for (i in 1:length(sel.tissues.names)){
        tissue.name = sel.tissues.names[i];
        tissue.name.nospace = sel.tissues.names.nospaces[i];
        output_table_file.name = paste(prefix.filename, sel.tissues.names.nospaces[i],".txt",sep="")
        # read the table with correlation info
        tissue.corr=read.table(output_table_file.name,h=T);
        colnames(tissue.corr) = c("index","genename","pearson.cor","pearson.pval","spearman.cor","spearman.pval");
        rownames(tissue.corr) = tissue.corr$genename;
        tissue.corr=merge(tissue.corr,gene.names,by.x="genename",by.y="Name")
        
        # positive correlations
        if(corr.type == 1){
            sig.genes = head(tissue.corr[order(tissue.corr$pearson.cor,decreasing=T),c("Description","genename","pearson.cor")], n_ranked_genes)
        }
        # negative correlations
        if(corr.type == -1){
            sig.genes = tail(tissue.corr[order(tissue.corr$pearson.cor,decreasing=T),c("Description","genename","pearson.cor")], n_ranked_genes)
        }
        # all correlations
        if(corr.type == 0){
            sig.genes = head(tissue.corr[order(abs(tissue.corr$pearson.cor),decreasing=T),c("Description","genename","pearson.cor")], n_ranked_genes)
        }    
        sig.genes = cbind(sig.genes,rep(tissue.name.nospace,nrow(sig.genes)))
        names(sig.genes)[4] = "tissue";
        cat(paste(tissue.name, nrow(sig.genes),sep=" "),sep="\n")
        all.sig.genes = rbind(all.sig.genes, sig.genes)
    }
    return (all.sig.genes)
}

# create a Matrix with genes occurring in three or more tisses as either pos or negatively correlated
# these are colored accordingly to the signal of the correlation
# usage: plotMatrixTopCorrelatedGenes(100)
plotMatrixTopCorrelatedGenes <-function(numTopGenes){
    # find the top #genes most correlated within each tissue
    all.sig.genes = filterTopCorrelatedGenes(sel.tissues.names, numTopGenes, "residualsregression/Regression.Residuals.", 0)
        #create a matrix with correlated genes and their occurring tissues
    tt = table(all.sig.genes$genename)
    # select only genes occurring in three or more samples
    genes.3ormoresamples = names(tt[tt>2])
    genes.tissues.withCorr = all.sig.genes[all.sig.genes$genename %in% genes.3ormoresamples,]
    corr.mat = matrix(ncol = length(sel.tissues.names), nrow=length(genes.3ormoresamples), "none");
    rownames(corr.mat) = gene.names[genes.3ormoresamples,c(2)]
    colnames(corr.mat) = sel.tissues.names
    for(i in seq(1:nrow(genes.tissues.withCorr))){
        g = as.character(genes.tissues.withCorr[i,1])
        t = as.character(chartr("."," ",genes.tissues.withCorr[i,4]))
        c = genes.tissues.withCorr[i,3]
        cat(paste(g,t,c,sep=" "),sep="\n")
        if(c > 0){corr.mat[g,t] = "pos"}
        if(c < 0){corr.mat[g,t] = "neg"}
    }
    # call the script for mutually exclusive plot of top 100 correlated genes across the different tissues
    source('basicOncoPrint.R')
    pdf("Matrix.CommonCorrelatedGenesAmongTissues.pdf",w=15,h=15, pointsize=12, useDingbats=FALSE)
        oncoPrint(corr.mat,tissue.colors.hash)
    dev.off()
}
######################################################################################
# Compute Correlation based on Ischemic Time of Sample without covariates
######################################################################################
runCorrIschemicSample <- function(selected.samples, output_table_file, min.output.corrvalue){
    # count genes with corr >= min.output.corrvalue
    num_genes = 0
    ## PREPARE GENE EXPRESSION DATA
    ## selects the gene expression for the input samples; first do a log2 transformation
    ## then do a quantile normalization
    rpkm.tissue = gene.rpkm[,colnames(gene.rpkm) %in% selected.samples];
    rpkm.tissue = log2(0.5+rpkm.tissue);
    rpkm = as.data.frame(normalize.quantiles(as.matrix(rpkm.tissue)));
    names(rpkm) = names(rpkm.tissue);
    rownames(rpkm) = rownames(rpkm.tissue);
    rm(rpkm.tissue)
    # retrieve the ischemic time for the respective samples
    ischemic.time=abs(as.numeric(sel.sample_attribs_subj[(sel.sample_attribs_subj$SAMPIDDOT %in% selected.samples),c("SMTSISCH")]))
    # compute the correlations
    pval.mat.res <- matrix(nrow=nrow(rpkm),ncol=6);
    for (i in 1:nrow(rpkm)){
        gname = as.character(rownames(rpkm[i,]))
        gexp = as.numeric(rpkm[i,]);
        gexpraw = 2^gexp -0.5
        
        if(mean(gexpraw) > 0.5){
            cc.pearson = cor.test(gexp,ischemic.time,method="pearson")
            cc.spearman = cor.test(gexp,ischemic.time,method="spearman")
            cc.pearson.cor = as.numeric(cc.pearson$estimate)
            cc.pearson.pval = as.numeric(cc.pearson$p.value)
            cc.spearman.cor = as.numeric(cc.spearman$estimate)
            cc.spearman.pval = as.numeric(cc.spearman$p.value)
            
            if(abs(cc.pearson.cor)>min.output.corrvalue & abs(cc.spearman.cor)>min.output.corrvalue){
                cat(paste(i,gname,cc.pearson.cor,cc.pearson.pval,cc.spearman.cor,cc.spearman.pval,sep=" "),sep="\n")
                num_genes = num_genes + 1
            }
            pval.mat.res[i,1]=i
            pval.mat.res[i,2]=as.character(gname)
            pval.mat.res[i,3]=cc.pearson.cor
            pval.mat.res[i,4]=cc.pearson.pval
            pval.mat.res[i,5]=cc.spearman.cor
            pval.mat.res[i,6]=cc.spearman.pval
        }
    }
    names(pval.mat.res) = c("index","genename","pearson.cor","pearson.pval","spearman.cor","spearman.pval")
    # output to text file
    write.table(pval.mat.res[!is.na(pval.mat.res[,3]),],output_table_file,sep="\t")
    cat(paste("Num genes with corr >=", min.output.corrvalue, ": ", num_genes),sep="\n")
}

# read the correlation info files from the different tissues and pick the top selected genes
# usage: tgl = retrieveTissueCorrs(sel.tissues.names, "Regression.Residuals.", 0.01, 0.6)
retrieveTissueCorrs <- function(sel.tissues.names,  prefix.filename, max_pval, min_corr){
    sel.tissues.names.nospaces = chartr(" ",".",sel.tissues.names)
    top.gene.lists = c()      
    # plot first tissue
    for(i in seq(1:length(sel.tissues.names))){
        tissue.name = sel.tissues.names[i]
        tissue.name.nospace = sel.tissues.names.nospaces[i]
        output_table_file.name = paste(prefix.filename, sel.tissues.names.nospaces[i],".txt",sep="")
        cat(output_table_file.name, sep="\n")
        # read the table with correlation info
        tissue.corr=read.table(output_table_file.name,h=T)
        colnames(tissue.corr) = c("index","genename","pearson.cor","pearson.pval","spearman.cor","spearman.pval")
        top.gene.ens = tissue.corr[abs(tissue.corr$pearson.cor)> min_corr & tissue.corr$pearson.pval < max_pval,c(2)]
        top.gene.name = as.character(gene.names[gene.names$Name %in% top.gene.ens,c(2)])
        cat(top.gene.name)
        top.gene.lists = c(top.gene.lists, list(top.gene.name))
    }
        return (top.gene.lists)
}

# Function to call the correlation with ischemic time for all tissues; receives as input a list of tissues
callCorrIschemicAllTissues <- function(sel.tissues.names, min.out.corr.val) {
    sel.tissues.names.nospaces = chartr(" ",".",sel.tissues.names)
    for (i in 1:length(sel.tissues.names)){
        tissue.name = sel.tissues.names[i]
        tissue.name.nospace = sel.tissues.names.nospaces[i]
        output_table_file.name = paste(sel.tissues.names.nospaces[i],".txt",sep="")
        cat(tissue.name,sep="\n")
        selected.tissue.samples = sample.matrix[sample.matrix$SMTSD==tissue.name,c("SAMPIDDOT")]
        runCorrIschemicSample(selected.tissue.samples, output_table_file.name, min.out.corr.val)
    }
}

#######################################
# Plot correlation between tissues in a matrix
#######################################
#usage: corr.mat = corrBetweenTissues(sel.tissues.names, "residualsregression/Regression.Residuals.")

#call function that creates a correlation matrix between all tissues
corrBetweenTissues <- function (sel.tissues.names, prefix.filename){
    t = 1
    corrArray <- matrix(nrow=length(sel.tissues.names)*(length(sel.tissues.names)-1)/2,ncol=3);
    for(i in seq(1,length(sel.tissues.names)-1)){
        for(j in (i+1):length(sel.tissues.names)){
            # tissue A
            tissueA.name = sel.tissues.names[i]
            tissueA.name.nospace = chartr(" ",".",tissueA.name)
            output_table_file.name = paste(prefix.filename, tissueA.name.nospace,".txt",sep="")
            tissueA.corr=read.table(output_table_file.name,h=T)
            colnames(tissueA.corr) = c("index","genename","pearson.cor","pearson.pval","spearman.cor","spearman.pval")
            # tissue B
            tissueB.name = sel.tissues.names[j]
            tissueB.name.nospace = chartr(" ",".",tissueB.name)
            output_table_file.name = paste(prefix.filename, tissueB.name.nospace,".txt",sep="")
            tissueB.corr=read.table(output_table_file.name,h=T)
            colnames(tissueB.corr) = c("index","genename","pearson.cor","pearson.pval","spearman.cor","spearman.pval")
            
            # intersect genes in common from 2 tissues
            tm = merge(tissueA.corr, tissueB.corr, by.x="genename",by.y="genename")
            corr.betweenTissues = cor(tm[,c(3)],tm[,c(8)],method="spearman")
            cat(paste( tissueA.name, tissueB.name , corr.betweenTissues,sep="\t"),sep="\n")
            corrArray[t,1] = tissueA.name
            corrArray[t,2] = tissueB.name
            corrArray[t,3] = corr.betweenTissues
            t = t + 1
        }
    }
    # output correlation matrix
    write.table(corrArray,"pairwiseMatrix.corrBetweenTissues.txt",quote=F,sep="\t")
    corrArray = read.table("pairwiseMatrix.corrBetweenTissues.txt",sep="\t")
    return (corrArray)
}

#plot heatmap
plotCorrelationMatrix <- function (corrArray, sel.tissues.names, outputfilename){
    library(gplots)
    library(RColorBrewer)
    
    S <- diag(length(sel.tissues.names))
    rownames(S) = sel.tissues.names
    colnames(S) = sel.tissues.names
    for(i in 1:nrow(corrArray)){
        a = as.character(corrArray[i, 1])
        b = as.character(corrArray[i, 2])
        c = as.numeric(corrArray[i, 3])
        S[a,b] = 1 - c
        S[b,a] = 1 - c
        S[a,a] = 0
        S[b,b] = 0
    }
        
    mycols=colorRampPalette(brewer.pal(9,"GnBu"))(15)
    pdf(outputfilename,w=20,h=20)
        par(oma=c(20,0,0,20))
        hcd = as.dendrogram(hclust(as.dist(S), method="average"))
        heatmap.2(S,col=redblue(100),scale="none",trace="none",Rowv=hcd,Colv=hcd,dendro="col", key=TRUE, keysize=0.8, symkey=TRUE, cexRow=1.4,cexCol=1.4,notecex=1,main="Spearman correlation between tissues")
    dev.off()
}

# Plot in a heatmap a correlation between different to reveal their similarity in terms of gene correlation
corr.mat = corrBetweenTissues(sel.tissues.names, "residualsregression/Regression.Residuals.")
plotCorrelationMatrix(corr.mat, sel.tissues.names, "matrix.correlBetweenTissues.pdf")






################################################################################
# FUNCTIONS FOR FUNCTIONAL ENRICHMENT ANALYISIS
################################################################################

# for the selected genes in the all.sig.genes
# create a list with respective the Entrez ids
createListSignificantGenesWithEntrezIds <-function (all.sig.genes){
    all.sig.genes.entrezids = list()
    tnames=c()
    i = 1
    for (ts in unique(all.sig.genes$tissue)){
        cat(ts, sep="\n")
        gl = all.sig.genes[all.sig.genes$tissue==ts,c("Description")]
        egl = bitr(gl, fromType="SYMBOL", toType="ENTREZID", annoDb="org.Hs.eg.db")
        egl.ids = egl[,c("ENTREZID")]
        if(length(egl.ids)>10){
            all.sig.genes.entrezids[[i]] = egl.ids
            i = i + 1
            tnames = c(tnames, ts)
        }
    }
    names(all.sig.genes.entrezids) = tnames
    return (all.sig.genes.entrezids)
}

# Function to test and plot enrichment analysis of KEGG pathways and Gene Ontoloy
# Positively and Negatively correlated genes are assessed separtely
# usage: plotEnrichment_KEGG_GO_CorrGenes(100, 0.01)
plotEnrichment_KEGG_GO_CorrGenes <- function(numTopGenes, MinPvalue){
    # list of positively, negativel and all correlated genes per tissue
    all.sig.genes = filterTopCorrelatedGenes(sel.tissues.names, numTopGenes, "residualsregression/Regression.Residuals.", 0)
    pos.sig.genes = filterTopCorrelatedGenes(sel.tissues.names, numTopGenes, "residualsregression/Regression.Residuals.", 1)
    neg.sig.genes = filterTopCorrelatedGenes(sel.tissues.names, numTopGenes, "residualsregression/Regression.Residuals.", -1)
    # convert common gene names to EntrezIds
    # Note that some genes are lost in these conversion
    all.sig.genes.entrezids = createListSignificantGenesWithEntrezIds(all.sig.genes)
    pos.cor.genes.entrezids = createListSignificantGenesWithEntrezIds(pos.cor.genes)
    neg.cor.genes.entrezids = createListSignificantGenesWithEntrezIds(neg.cor.genes)

    # Enrichment function from Clusterprofiler for GO analysis
    kegg.pos <- compareCluster(geneCluster = pos.cor.genes.entrezids, fun = "enrichKEGG", organism="human", pvalueCutoff=MinPvalue, use_internal_data = TRUE)
    go.pos <- compareCluster(geneCluster = pos.cor.genes.entrezids, fun = "enrichGO", organism="human", pvalueCutoff=MinPvalue)
    kegg.neg <- compareCluster(geneCluster = neg.cor.genes.entrezids, fun = "enrichKEGG", organism="human", pvalueCutoff=MinPvalue, use_internal_data = TRUE)
    go.neg <- compareCluster(geneCluster = neg.cor.genes.entrezids, fun = "enrichGO", organism="human", pvalueCutoff=MinPvalue)

    # plot GO and KEGG Enrichment plots
    pdf("GO.negative.cor.pdf",w=12,h=10)
        print(plot(go.neg, type="dot",showCategory=3))
    dev.off()    
    pdf("KEGG.negative.cor.pdf",w=6,h=6)
        print(plot(kegg.neg, type="dot",showCategory=NULL))
    dev.off()
    pdf("GO.positive.cor.pdf",w=9,h=8)
        print(plot(go.pos, type="dot",showCategory=3))
    dev.off()    
    pdf("KEGG.positive.cor.pdf",w=8,h=8)
        print(plot(kegg.pos, type="dot",showCategory=NULL))
    dev.off()
}

#######################################
# Map gene Correlation with PMI on KEGG pathways
# after compareCluster analysis
#######################################
# for a given tissue creates a list of genes with a certain score
# in this case the score corresponds to the correlation values with ischemic time
# gene names are then converted to EntrezIDs
# the geneList returns a list with names as the entrez ids and the values as the correlation score
# usage: adiposesubcutaneous.corr = geneListCorrelationByTissue("Adipose - Subcutaneous")
geneListCorrelationByTissue <- function(tissue.name){
    cat(tissue.name, sep="\n")
    tissue.name.nospace = chartr(" ",".",tissue.name)
    output_table_file.name = paste(prefix.filename, tissue.name.nospace,".txt",sep="")
    # read the table with correlation info
    tissue.corr=read.table(output_table_file.name,h=T);
    colnames(tissue.corr) = c("index","genename","pearson.cor","pearson.pval","spearman.cor","spearman.pval");
    rownames(tissue.corr) = tissue.corr$genename;
    tissue.corr=merge(tissue.corr,gene.names,by.x="genename",by.y="Name")
    egl = bitr(tissue.corr$Description, fromType="SYMBOL", toType="ENTREZID", annoDb="org.Hs.eg.db")
    egl = merge(tissue.corr, egl, by.x="Description",by.y="SYMBOL")
    geneList = egl[,c("pearson.cor")]
    names(geneList)= egl[,c("ENTREZID")]
    return (geneList)
}

#usage: plotKEGGPathwayBasedCorrelationByTissue("Whole Blood", "hsa04920")
plotKEGGPathwayBasedCorrelationByTissue <- function(tissue.name, pathwayID){
    geneList = geneListCorrelationByTissue(tissue.name)
    pathPlot <- pathview(gene.data  = geneList, pathway.id = pathwayID,species= "hsa", limit= list(gene=max(abs(geneList)), cpd=1), low = list(gene = "blue", cpd = "blue"))
}

###########################################################
# Comparison of Ischemic time with RIN
###########################################################
# SMRIN: The RNA Integrity Number, a basic measure of the quality of RNA isolated
plotPMI_wrt_RIN <- function(){
    # Several plots with PMI and RIN relation
    pdf("plot.ischemictime_vs_RIN.allsamples.pdf")
        plot(sel.sample.subj.covars$SMTSISCH,sel.sample.subj.covars$SMRIN,main="All samples, R:-0.32 (all values); R:-0.27 (pos. values)",xlab="Ischemic time", ylab="RIN",col="blue")
    dev.off()

    pdf("smoothscatter.ischemictime_vs_RIN.allsamples.pdf")
        smoothScatter(sel.sample.subj.covars$SMTSISCH,sel.sample.subj.covars$SMRIN,main="All samples,R:-0.32 (all values); R:-0.27 (pos. values)",xlab="Ischemic time", ylab="RIN")
    dev.off()

    # for blood samples
    blood.SM_attribs = sel.sample.subj.covars[sel.sample.subj.covars$SMTSD=="Whole Blood",]
    cor(blood.SM_attribs$SMTSISCH,blood.SM_attribs$SMRIN,use="pairwise.complete.obs")
    
    pdf("plot.ischemictime_vs_RIN.Blood.pdf")
        plot(sel.sample.subj.covars[sel.sample.subj.covars$SMTSD=="Whole Blood",c("SMTSISCH")], sel.sample.subj.covars[sel.sample.subj.covars$SMTSD=="Whole Blood",c("SMRIN")],main="Blood samples, R:-0.197 (all values); R:-0.08 (positive values)",xlab="Ischemic time", ylab="RIN",col=as.character(tissue.colors.hash[tissue.colors.hash$tissue=="Whole Blood",c("color")]))
    dev.off()

    # make the correlation of ischemic time and RIN for all tissues: Decompose by tissue 
    tissue.cor.ischemic.rin = matrix(0,length(sel.tissues.names),2)
    for(i in seq(1,length(sel.tissues.names))){
        tissue.name = sel.tissues.names[i]
        pc = cor(sel.sample.subj.covars[sel.sample.subj.covars$SMTSD==tissue.name,c("SMTSISCH","SMRIN")],use="pairwise.complete.obs")
        tissue.cor.ischemic.rin[i,1] = as.character(tissue.name)
        tissue.cor.ischemic.rin[i,2] = pc[1,2]
    }
    write.table(tissue.cor.ischemic.rin, "tissue.cor.ischemic.rin.txt",row.names=F,col.names=F,sep="\t")
    tissue.cor.ischemic.rin=read.table("tissue.cor.ischemic.rin.txt")
    tissue.cor.ischemic.rin = tissue.cor.ischemic.rin[order(tissue.cor.ischemic.rin$V2),]
    tissue.cor.ischemic.rin = merge(tissue.cor.ischemic.rin, tissue.colors.hash,by="V1",by.y="tissue")
    tissue.cor.ischemic.rin = tissue.cor.ischemic.rin[order(tissue.cor.ischemic.rin$V2),]

    pdf("barplot.tissue.correlation.ischemic.rin.pdf",w=12,h=12)
        par(mar=c(18,4.1,4.1,2.1))
        barplot(tissue.cor.ischemic.rin$V2,ylim=c(-1,0.1),names=tissue.cor.ischemic.rin$V1,las=2,main="Correlation of Ischemic time with RIN",col=as.character(tissue.cor.ischemic.rin$color))
    dev.off()
}

##################################################################
# Compare premortem and postmortem blood samples
# function that calculates the differential expression between
# DIFFERENTIAL EXPRESSION PRE and POST mortem blood samples
#######################################################################################
library("edgeR")

load("RWorkspace.to.run.DEblood.R")

run.DiffExpr.Pre.Post.Mortem <- function(interval){
 
    print(interval)
    sel.blood.POSTMORTEM.interval <- pmi.sel.blood.POSTMORTEM[pmi.sel.blood.POSTMORTEM$SMTSISCH > interval[1] & pmi.sel.blood.POSTMORTEM$SMTSISCH <= interval[2],c("SAMPIDDOT")]
    print(length(sel.blood.POSTMORTEM.interval))
    print(length(sel.blood.PREMORTEM))
    # GROUP: pre and post mortem
    group <- factor(c(rep(1,length(sel.blood.PREMORTEM)),rep(2,length(sel.blood.POSTMORTEM.interval))))
    print(length(group))
    counts <- cbind(counts.in.blood[,colnames(counts.in.blood) %in% sel.blood.PREMORTEM], counts.in.blood[,colnames(counts.in.blood) %in% sel.blood.POSTMORTEM.interval])
    print(dim(counts))
    
    covars.list = c("SMRIN", "MHCANCERNM", "HGHT", "SMATSSCR", "WGHT", "BMI", "AGE", "ETHNCTY","TRDNISCH")
    all.covars.list = sel.sample.subj.covars[colnames(counts), names(sel.sample.subj.covars) %in% as.character(covars.list)]
    all.covars.list = cbind(all.covars.list, as.numeric(sel.sample.subj.covars[colnames(counts), c("SMCENTER")]))
    all.covars.list = cbind(all.covars.list, as.numeric(sel.sample.subj.covars[colnames(counts), c("SMTSTPTREF")]))
    all.covars.list = cbind(all.covars.list, as.numeric(sel.sample.subj.covars[colnames(counts), c("SMNABTCHT")]))
    all.covars.list = cbind(all.covars.list, as.numeric(sel.sample.subj.covars[colnames(counts), c("COHORT")]))
    all.covars.list = cbind(all.covars.list, as.numeric(sel.sample.subj.covars[colnames(counts), c("GENDER")]))
    all.covars.list = cbind(all.covars.list, as.numeric(sel.sample.subj.covars[colnames(counts), c("DTHHRDY")]))
    names(all.covars.list)[c(10:15)] = c("SMCENTER","SMTSTPTREF","SMNABTCHT", "COHORT", "GENDER","DTHHRDY")
    # SMATSSCR had all NA values
    all.covars.list = all.covars.list[,-c(1)]
    all.covars.list = cbind(all.covars.list,group)
    # remove NA
    all.covars.list[is.na(all.covars.list)] = -1
    
    all.covars.list$GENDER = as.factor(as.character(all.covars.list$GENDER))
    all.covars.list$COHORT = as.factor(as.character(all.covars.list$COHORT))
    all.covars.list$DTHHRDY = as.factor(as.character(all.covars.list$DTHHRDY))
    all.covars.list$SMCENTER = as.factor(as.character(all.covars.list$SMCENTER))
    all.covars.list$SMNABTCHT = as.factor(as.character(all.covars.list$SMNABTCHT))
    all.covars.list$SMTSTPTREF = as.factor(as.character(all.covars.list$SMTSTPTREF))
    all.covars.list$ETHNCTY = as.factor(as.character(all.covars.list$ETHNCTY))
    all.covars.list$MHCANCERNM = as.factor(as.character(all.covars.list$MHCANCERNM))
    
    all.covars.list$SMRIN <- as.factor(as.character(cut(all.covars.list$SMRIN, c(5,7,8,9,10))))
    all.covars.list$AGE <- as.factor(as.character(cut(all.covars.list$AGE, c(20,30,40,50,60,70), include.lowest=T)))

    print(dim(all.covars.list))
    print(sapply(all.covars.list, class))
    
    print(all.equal(colnames(counts), rownames(all.covars.list)))
    
    #design with covariates
    design <- model.matrix(~ SMRIN + AGE + ETHNCTY + MHCANCERNM + SMCENTER + SMTSTPTREF + SMNABTCHT + GENDER + group, all.covars.list)
    
    print(head(design))
    
    # read counts
    y <- DGEList(counts=counts, genes=keep.ensids)
    # normalization TMM
    y <- calcNormFactors(y)
    # interested on the difference between pre and post mortem samples
    y <- estimateGLMCommonDisp(y,design)
    y <- estimateGLMTrendedDisp(y,design)
    y <- estimateGLMTagwiseDisp(y,design)
    
    fit <- glmFit(y,design)
    #  Note that we can omit the coef argument to glmLRT because the treatment effect is the last coefficient in the model.
    lrt <- glmLRT(fit)
    FDR <- p.adjust(lrt$table$PValue, method="BH")
    degs = cbind(lrt$table,FDR)
    degs = cbind(degs,readcnt.blood[rownames(degs),c(1,2)])
    degs = degs[order(degs$FDR,decreasing=F),]
    
    write.table(degs, paste("genelist.pre_mortem.vs.post_mortem", paste(as.character(interval[1]), as.character(interval[2]), sep="_"), "txt", sep = "."), row.names=F, col.names=T, quote=F, sep = "\t")
    
    # diff expressed genes in a design with covariates
    de.blood.fc.2.post = degs[degs$logFC>log2(2) & degs$FDR < 0.01,c("Name","Description")]
    de.blood.fc.2.pre = degs[degs$logFC < -log2(2) & degs$FDR < 0.01,c("Name","Description")]
    #add biotype
    de.blood.fc.2.pre = merge(de.blood.fc.2.pre,bt,by.x="Name",by.y="id")
    write.table(de.blood.fc.2.pre, paste("genelist.pre_mortem.fc", as.character(2), paste(as.character(interval[1]), as.character(interval[2]), sep="_"), "txt", sep = "."),row.names=F, col.names=F,quote=F, sep = "\t")
    de.blood.fc.2.post = merge(de.blood.fc.2.post,bt,by.x="Name",by.y="id")
    write.table(de.blood.fc.2.post, paste("genelist.post_mortem.fc", as.character(2), paste(as.character(interval[1]), as.character(interval[2]), sep="_"), "txt", sep = "."),row.names=F, col.names=F,quote=F, sep = "\t")
    
    
    de.blood.fc.3.post = degs[degs$logFC>log2(3) & degs$FDR < 0.01,c("Name","Description")]
    de.blood.fc.3.pre = degs[degs$logFC < -log2(3) & degs$FDR < 0.01,c("Name","Description")]
    #add biotype
    de.blood.fc.3.pre = merge(de.blood.fc.3.pre,bt,by.x="Name",by.y="id")
    write.table(de.blood.fc.3.pre, paste("genelist.pre_mortem.fc", as.character(3), paste(as.character(interval[1]), as.character(interval[2]), sep="_"), "txt", sep = "."),row.names=F, col.names=F,quote=F, sep = "\t")
    de.blood.fc.3.post = merge(de.blood.fc.3.post,bt,by.x="Name",by.y="id")
    write.table(de.blood.fc.3.post, paste("genelist.post_mortem.fc", as.character(3), paste(as.character(interval[1]), as.character(interval[2]), sep="_"), "txt", sep = "."),row.names=F, col.names=F,quote=F, sep = "\t")
    
    l = list( c(nrow(de.blood.fc.2.pre), sum(de.blood.fc.2.pre$bt == "protein_coding"), nrow(de.blood.fc.2.post), sum(de.blood.fc.2.post$bt == "protein_coding")), c(nrow(de.blood.fc.3.pre), sum(de.blood.fc.3.pre$bt == "protein_coding"), nrow(de.blood.fc.3.post), sum(de.blood.fc.3.post$bt == "protein_coding")))
    
    return(l)
}

intervals <- cbind(c(0,0,406,635,867),c(1401,406,635,867,1401))

for(i in 1:nrow(intervals)){
    write.table(pmi.sel.blood.POSTMORTEM[pmi.sel.blood.POSTMORTEM$SMTSISCH > intervals[i,1] & pmi.sel.blood.POSTMORTEM$SMTSISCH <= intervals[i,2],c("SAMPIDDOT")],  file = paste("samples.Post.mortem.interval.", i, ".txt", sep=""), quote=F, col.names=F)
}
write.table(sel.blood.PREMORTEM,  file = "samples.Pre.mortem.interval.txt", quote=F, col.names=F)

out.table.fc.2 <- c()
out.table.fc.3 <- c()
for(i in 1:nrow(intervals)){
    run <- run.DiffExpr.Pre.Post.Mortem( intervals[i,] )
    out.table.fc.2 <- rbind( out.table.fc.2, run[[1]] )
    out.table.fc.3 <- rbind( out.table.fc.3, run[[2]] )
}

colnames(out.table.fc.2) <- c("up.PRE.All","up.PRE.PT","up.POST.All", "up.POST.PT")
rownames(out.table.fc.2) <- c("All", ">0 & <=406", ">406 & <=635", ">635 & <=867", ">867 & <=1401")
write.table(out.table.fc.2, "out.table.fc.2.txt",row.names=T, col.names=T,quote=F, sep = "\t")

colnames(out.table.fc.3) <- c("up.PRE.All","up.PRE.PT","up.POST.All", "up.POST.PT")
rownames(out.table.fc.3) <- c("All", ">0 & <=406", ">406 & <=635", ">635 & <=867", ">867 & <=1401")
write.table(out.table.fc.3, "out.table.fc.3.txt",row.names=T, col.names=T,quote=F, sep = "\t")


save(list = ls(), file = "output.RData")

#nohup R CMD BATCH runDEblood_3.R &


t1.up.post <- read.table("genelist.post_mortem.fc.2.0_406.txt", h=F, sep="\t", stringsAsFactors=F)
t1.up.pre <- read.table("genelist.pre_mortem.fc.2.0_406.txt", h=F, sep="\t", stringsAsFactors=F)
t2.up.post <- read.table("genelist.post_mortem.fc.2.406_635.txt", h=F, sep="\t", stringsAsFactors=F)
t2.up.pre <- read.table("genelist.pre_mortem.fc.2.406_635.txt", h=F, sep="\t", stringsAsFactors=F)
t3.up.post <- read.table("genelist.post_mortem.fc.2.635_867.txt", h=F, sep="\t", stringsAsFactors=F)
t3.up.pre <- read.table("genelist.pre_mortem.fc.2.635_867.txt", h=F, sep="\t", stringsAsFactors=F)
t4.up.post <- read.table("genelist.post_mortem.fc.2.867_1401.txt", h=F, sep="\t", stringsAsFactors=F)
t4.up.pre <- read.table("genelist.pre_mortem.fc.2.867_1401.txt", h=F, sep="\t", stringsAsFactors=F)

# PLOT FOR DIFFERENTIAL EXPRESSION BETWEEN TIME INTERVALS IN BLOOD
dat <- read.table(text = "Interval,Pre,Post_Ti,Pre_Previous,Pre_New, Post_Previous, Post_new
>0 & <=406,547,1039,0,547,0,1039
>406 & <=635,1907,1940,533,1374,939,1001
>635 & <=867,1767,1973,1683,84,1760,213
>867 & <=1401,244,933,240,4,913,20", header= TRUE, sep = ",")


pdf("DiffExpression.Intervals.Blood.pdf",pointsize=16, useDingbats=FALSE, w=14,h=12)


vals = t(dat[,c(2,3)])
barplot(vals, col=c("#BDF48C","#BBBCBC"), axes=F, axisnames=F, yaxp=c(0,1,2), las=1,beside=T)


vals = t(dat[,c(4,6)])
for (i in 1:ncol(vals)){
    xx = vals
    xx[,-i] <- NA
    colnames(xx)[-i] <- NA
    if (i < ncol(vals)){
        barplot(xx,col=c("#68B32F","#6C6D6D"), add=T, axes=F,beside=T)
    }else{
        barplot(xx,col=c("#68B32F","#6C6D6D"), add=T,beside=T, names =c("G1","G2","G3","G4"),las=1,yaxp=c(0,2000, 5) )
    }
}

dev.off()

###########################################################
## GO ANALYSIS FOR PRE AND POST MORTEM Diff. Exp. GENES
############################################################
enrichmentAnalysis_GO_KEGG_DifferentialBlood <- function(){
    library("DOSE")
    library("clusterProfiler")
    library(KEGG.db)
    library(pathview)

    # Universe of genes corresponds to all gene in the annotation file
    background.geneList = bitr(readcnt.blood[,c("Description")], fromType="SYMBOL", toType="ENTREZID", annoDb="org.Hs.eg.db")
    #Warning message:
    #In bitr(readcnt.blood[, c("Description")], fromType = "SYMBOL",  : 46.54% o
    eg = bitr(de.blood.fc2.post$Description, fromType="SYMBOL", toType="ENTREZID", annoDb="org.Hs.eg.db")
    eg = merge(eg,degs,by.x="SYMBOL",by.y="Description")
    gene <- eg$ENTREZID
    # get GO classification of all genes; may skip this
    ego.bp <- groupGO(gene=gene,organism="human",ont="BP",level=3,readable=TRUE)
    ego.mf <- groupGO(gene=gene,organism="human",ont="MF",level=3,readable=TRUE)
    ego.cc <- groupGO(gene=gene,organism="human",ont="CC",level=3,readable=TRUE)
    kegg <- enrichKEGG(gene = gene,organism = "human", pvalueCutoff = 0.01,readable= TRUE, use_internal_data = TRUE)
    #head(summary(kegg))
    #head(summary(ego.bp))
    pdf("KEGG.barplot.de_fc3.PremortemUP.pdf",w=18,h=14)
        barplot(kegg)
    dev.off()
    pdf("KEGG.de_fc3.PremortemUP.pdf",w=18,h=14)
        cnetplot(kegg)
    dev.off()

    # VISUALIZATION: other formats
    enrichMap(ego.cc)
    barplot(ego.cc, drop=TRUE, showCategory=20)
    cnetplot(ego, categorySize="pvalue", foldChange=eg$logFC)

    # save files     
    setwd("Blood_pre_post_comparison/GO/")
    write.table(summary(ego.bp)[c(1,7)],"REVIGO.BP.de_fc3.PremortemUP.txt",quote=F,row.names=F,col.names=F,sep="\t")
    write.table(summary(ego.mf)[c(1,7)],"REVIGO.MF.de_fc3.PremortemUP.txt",quote=F,row.names=F,col.names=F,sep="\t")
    write.table(summary(ego.cc)[c(1,7)],"REVIGO.CC.de_fc3.PremortemUP.txt",quote=F,row.names=F,col.names=F,sep="\t")
    write.table(summary(ego.bp),"GO.BP.de_fc3.PremortemUP.txt",quote=F,row.names=F,col.names=F,sep="\t")
    write.table(summary(ego.mf),"GO.MF.de_fc3.PremortemUP.txt",quote=F,row.names=F,col.names=F,sep="\t")
    write.table(summary(ego.cc),"GO.CC.de_fc3.PremortemUP.txt",quote=F,row.names=F,col.names=F,sep="\t")
    write.table(summary(kegg),"KEGG.de_fc3.PremortemUP.txt",quote=F,row.names=F,col.names=F,sep="\t")
}
    
   
###########################################################################################################
# CLUSTERING AND MULTIDIMENSIONAL SCALING of blood samples and coloring according to pre/post mortem status
# hcClustBlood
###########################################################################################################
samples.tissueA = sel.blood.PREMORTEM
samples.tissueB = sel.blood.POSTMORTEM
blood.premortem.times = sel.sample_attribs_subj[samples.tissueA,c("SMTSISCH","TRISCHD", "TRDNISCH")]
blood.postmortem.times = sel.sample_attribs_subj[samples.tissueB,c("SMTSISCH","TRISCHD", "TRDNISCH")]
samples.blood.collectafterdeath = rownames(blood.postmortem.times[blood.postmortem.times$SMTSISCH > blood.postmortem.times$TRISCHD,])
samples.tissueB = setdiff(samples.tissueB, samples.blood.collectafterdeath)


hcClustBlood(sel.blood.PREMORTEM, sel.blood.POSTMORTEM)

# Hierarchical clustering and MDS for PRE and POST MORTEM BLOOD samples
hcClustBlood <- function (samples.tissueA, samples.tissueB){
    library(gplots)
    library(MASS)
    library(preprocessCore)
    # selects the gene expression for the input samples, converts to log2 scale and quantile normalizes the expression log2 transformed values
    rpkm.tissueA = gene.rpkm[,colnames(gene.rpkm) %in% samples.tissueA]
    rpkm.tissueB = gene.rpkm[,colnames(gene.rpkm) %in% samples.tissueB]
    rpkm.tissueA = log2(1+rpkm.tissueA);
    rpkm.tissueB = log2(1+rpkm.tissueB);
    rpkm.tissueA = as.data.frame(normalize.quantiles(as.matrix(rpkm.tissueA)));
    rpkm.tissueB = as.data.frame(normalize.quantiles(as.matrix(rpkm.tissueB)));
    rpkm.blood = gene.rpkm[,c(1,2)]
    rpkm.blood = cbind(rpkm.blood, rpkm.tissueA)
    rpkm.blood = cbind(rpkm.blood, rpkm.tissueB)
    names(rpkm.blood) = c("name", "description", as.character(samples.tissueA), as.character(samples.tissueB))
        
    # color scheme where samples are colored by ischemic time interval
    isch.time = sel.sample_attribs_subj[samples.tissueB,c("SMTSISCH")]
    isch.col = rep("gray",length(isch.time))
    i = 1
    for(it in isch.time){
        if(it < 0) {isch.col[i] = "#FFFFFF"}
        if(it < 60) {isch.col[i] = "green"}
        if(it >= 60 & it < 240) {isch.col[i] = "greenyellow"}
        if(it >= 240 & it < 360) {isch.col[i] = "yellow"}
        if(it >= 360 & it < 900) {isch.col[i] = "orange"}
        if(it >= 900) {isch.col[i] = "red"}
        i = i + 1
    }
    bloodcols = c(rep("gray",length(samples.tissueA)),isch.col)
    
    rpkm.blood = cbind(rpkm.tissueA, rpkm.tissueB)
    rm(samples.tissueA)
    rm(samples.tissueB)
    # distance matrix 
    d.s <- as.dist(1-cor(rpkm.blood, method="pearson"))
    # hierarchical clustering 
    hc.s <- hclust(d.s, method="average")

    #hclust pearson and complete
    pdf("HC.expression.blood.pre_postmortem.pdf",w=14,h=14)
        heatmap.2(as.matrix(d.s),col=redblue(40),scale="none",trace="none",as.dendrogram(hc.s),labRow="",labCol="",main="",ColSideColors=bloodcols,Rowv=as.dendrogram(hc.s), dendrogram="col",density.info="none",key=F)
        legend("topleft",  c("Pre-Mortem", "< 1 Hr",">=1 Hr, <4 Hr",">=4Hr, <6Hr",">=6Hr, <15Hr",">=15Hr"), cex=1.2, col=c("gray","green","greenyellow","yellow","orange","red"),lty=1, lwd=5,bty="n")
    dev.off()
    #MDS
    ord<- isoMDS(d.s,k=3) # for 2 dimensions
    pdf("MDS.expression.blood.pre_postmortem.pc1_pc2.noCollectedafterdeath.pdf",pointsize=8, useDingbats=FALSE, w=8,h=8)
        plot(ord$points,  type="n",xlab="Coordinate 1",ylab="Coordinate 2",main="")
        points(ord$points,bg=bloodcols,col=rep("black",length(bloodcols)),pch=21,cex=1.7)
        legend("topleft",  c("Pre-Mortem", "< 1 Hr",">=1 Hr, <4 Hr",">=4Hr, <6Hr",">=6Hr, <15Hr",">=15Hr"), cex=1.2, col=c("gray","green","greenyellow","yellow","orange","red"),lty=1, lwd=5,bty="n")
    dev.off()
}


#####################################################################################################
# plot MDS with different colors according to the covariate HARDY SCALE OR VENTILATOR
#####################################################################################################

plotMDS_HardyScaleVentilator <- function(){
    pointTypes = c(rep(21,length(samples.tissueA)), rep(23,length(samples.tissueB)))
    # hardy scale
    var="DTHHRDY"
    hardy.values = c(sel.sample_attribs_subj[sel.sample_attribs_subj$SAMPIDDOT %in% sel.blood.PREMORTEM, c(var)], sel.sample_attribs_subj[sel.sample_attribs_subj$SAMPIDDOT %in% sel.blood.POSTMORTEM, c(var)])
    hardy.color = hardy.values
    bugn=c('#edf8fb','#b2e2e2','#66c2a4','#2ca25f','#006d2c')
    hardy.color[hardy.color==0] = bugn[1]
    hardy.color[hardy.color==1] = bugn[2]
    hardy.color[hardy.color==2] = bugn[3]
    hardy.color[hardy.color==3] = bugn[4]
    hardy.color[hardy.color==4] = bugn[5]
    hardy.color[is.na(hardy.color)] = "gray"
    pdf("MDS.expression.blood.pre_postmortem.pc1_pc2.HARDY.noCollectedafterdeath.pdf",pointsize=8, useDingbats=FALSE, w=8,h=8)
        plot(ord$points,  type="n",xlab="Coordinate 1",ylab="Coordinate 2",main="Hardy Scale")
        points(ord$points,bg = hardy.color ,col=rep("black",length(bloodcols)),pch=pointTypes,cex=1.7)
        legend("topleft",  c("Pre-Mortem", "Post_mortem"), cex=1.2, pch=c(21,23),bty="n")
        legend("topright",  c("0", "1", "2","3", "4", "NA"), col=c(bugn,"gray"), cex=1.2, pch=19,bty="n")
    dev.off()
    
    # ventilator or not
    var="DTHVNT"
    ventilator.values = rbind(sel.sample_attribs_subj[sel.sample_attribs_subj$SAMPIDDOT %in% sel.blood.PREMORTEM, c("SAMPIDDOT", var)], sel.sample_attribs_subj[sel.sample_attribs_subj$SAMPIDDOT %in% sel.blood.POSTMORTEM, c("SAMPIDDOT", var)])
    ventilator.color = ventilator.values
    ventilator.color[is.na(ventilator.color$DTHVNT), "DTHVNT"] = "gray"
    ventilator.color[ventilator.color$DTHVNT==0,"DTHVNT"] = "orange"
    ventilator.color[ventilator.color$DTHVNT==1, "DTHVNT"] = "darkred" 

    pdf("MDS.expression.blood.pre_postmortem.pc1_pc2.VENTILATOR.noCollectedafterdeath.pdf",pointsize=8, useDingbats=FALSE, w=8,h=8)
        plot(ord$points,  type="n",xlab="Coordinate 1",ylab="Coordinate 2",main="Individuals in ventilator")
        points(ord$points,bg =ventilator.color$DTHVNT ,col=rep("black",length(bloodcols)),pch=pointTypes,cex=1.7)
        legend("topleft",  c("Pre-Mortem", "Post_mortem"), cex=1.2, pch=c(21,23),bty="n")
        legend("topright",  c("Ventilator", "No Ventilator", "NA"), col=c("darkred","orange", "gray"), cex=1.2, pch=19,bty="n")
    dev.off()
}




########################################################################################################################
# EXTRACT GENE FEATURE
########################################################################################################################
# gene descriptions
gene.descriptions=read.delim("mart_export.txt",sep="\t",h=T)
# get annotation file from Gencode
cat gencode.v19.annotation.gtf |awk 'BEGIN{OFS="\t";} $3=="exon" {print $1,$4-1,$5,$10}' | sed 's/";//'  | sed 's/"//'  | sortBed | mergeBed -i - | gzip > gencode_v19_exon_merged.bed.gz
# number of exons per gene
zcat gencode_v19_exon_merged.bed.gz | cut -f4 | sort | uniq -c | awk '{print $2"\t"$1}'  > number.exons.pergene.txt
awk '{split($1,a,"."); print a[1]"\t"$0}' number.exons.pergene.txt > number.exons.ensemblids.txt
# length of CDS of the gene
zcat gencode_v19_exon_merged.bed.gz | awk '{d=$3-$2; lg[$4]=lg[$4]+d}END{for (g in lg){print g"\t"lg[g]}}' > length.CDS.pergene.txt
# length of whole gene
cat gencode.v19.annotation.gtf | awk '{if($3=="gene"){d=$5-$4; print $10"\t"d}}' | sed 's/";//g'  | sed 's/"//g' > length.wholegene.pergene.txt
# biotype of the gene
cat gencode.v19.annotation.gtf | awk '{if($3=="gene")print $10"\t"$14}'  | sed 's/";//g'  | sed 's/"//g' > biotype.gene.txt
# GC content
awk '{print $1"\t"$3}' mart_export.txt | sort | uniq > GCcontent.genes.txt

# load a merge gene features in R
prepareGeneFeaturesTable <- function(){
    #chromossomes for the genes
    chroms = read.table("gene.ids.names.perChrom.txt",h=T)
    # merge everything in R
    a=read.table("number.exons.ensemblids.txt")
    bt=read.table("/home/pferreira/Projects/genomeFiles/Gencode/geneFeatures/biotype.gene.txt")
    names(bt)=c("id","bt")
    lnCDS=read.table("length.CDS.pergene.txt")
    lnWG=read.table("length.wholegene.pergene.txt")
    a=merge(a,gc,by.x="V1",by.y="V1")
    names(a)=c("gene","id","nexons","GC")
    a=merge(a,bt,by.x="id",by.y="V1")
    names(a)[5]="biotype"
    a=merge(a,lnCDS,by.x="id",by.y="V1")
    names(a)[6]="lenCDS"
    a=merge(a,lnCDS,by.x="id",by.y="V1")
    names(a)[7]="lenWholeGene"
    write.table(a[,-c(2)],"gene.features.txt")
    gene.feat = read.table("gene.features.txt")
    rownames(gene.feat) = gene.feat$id
    return (gene.feat)
}

# This function receives a tissue name a a vector with several genes features
# it then plots the correlation values (gene expression with ischemic time) with each of the gene features
# it allows us to understand if correlation with ischemic time is related with one of these gene features
#usage rescorr = plotGeneFeaturesByTissue(tissue.name, gene.feat)

plotGeneFeaturesByTissue <- function(tissue.name, gene.features){
    prefix.filename = "/home/pferreira/Projects/GTEx2/ischemicTimeStudy/LinearRegression/residualsregression/Regression.Residuals."
    tissue.names.nospaces = chartr(" ",".", tissue.name);   
    output_table_file.name = paste(prefix.filename, tissue.names.nospaces,".txt",sep="")
    res = list()
    
    # read the table with correlation info
    tissue.corr=read.table(output_table_file.name,h=T);
    colnames(tissue.corr) = c("index","genename","pearson.cor","pearson.pval","spearman.cor","spearman.pval");
    rownames(tissue.corr) = tissue.corr$genename;
    tissue.corr = tissue.corr[order(tissue.corr$pearson.cor),]
    
    output_table_file.name = paste("GeneFeatures.", tissue.names.nospaces,".png",sep="")
    
    png(output_table_file.name,w=1400,h=1400)
    par(mfrow=c(2,2))
    corr.nexons = cor.test(tissue.corr$pearson.cor, gene.feat[rownames(tissue.corr),c("nexons")],use="na.or.complete");
    corr.gc = cor.test(tissue.corr$pearson.cor, gene.feat[rownames(tissue.corr),c("GC")],use="na.or.complete");
    corr.lenCDS = cor.test(tissue.corr$pearson.cor, gene.feat[rownames(tissue.corr),c("lenCDS")],use="na.or.complete");
    corr.lenGene = cor.test(tissue.corr$pearson.cor, gene.feat[rownames(tissue.corr),c("lenGene")],use="na.or.complete");
    corr.biotype = cor.test(tissue.corr$pearson.cor, as.numeric(gene.feat[rownames(tissue.corr),c("biotype")]),use="na.or.complete");
    res[1] = as.numeric(sprintf("%.2f", corr.nexons$estimate[[1]]));
    res[2] = as.numeric(sprintf("%.2f",corr.gc$estimate[[1]]));
    res[3] = as.numeric(sprintf("%.2f",corr.lenCDS$estimate[[1]]));
    res[4] = as.numeric(sprintf("%.2f",corr.lenGene$estimate[[1]]));
    res[5] = as.numeric(sprintf("%.2f",corr.biotype$estimate[[1]]));
    
    tit = paste("Num Exons; R = ", res[1])
    plot(tissue.corr$pearson.cor, gene.feat[rownames(tissue.corr),c("nexons")],main=tit,ylab="Num Exons",xlab="correlation with ischemic time")
    tit = paste("GC Content; R = ", res[1])
    plot(tissue.corr$pearson.cor, gene.feat[rownames(tissue.corr),c("GC")],main=tit,xlab="correlation with ischemic time", ylab="GC Content")
    tit = paste("CDS Length; R = ", res[1])
    plot(tissue.corr$pearson.cor, gene.feat[rownames(tissue.corr),c("lenCDS")],main=tit,xlab="correlation with ischemic time", ylab="CDS length")
    tit = paste("Gene Length; R = ", res[1])
    plot(tissue.corr$pearson.cor, gene.feat[rownames(tissue.corr),c("lenGene")],main=tit,xlab="correlation with ischemic time",ylab="Gene length")
    dev.off()
    return (res);
}


# wrapper for plotGeneFeaturesByTissue function
# it keeps all the 4 correlation values per tissue in a single table
#usage: plotGeneFeaturesByTissueWrapper(sel.tissues.names, gene.feat)
plotGeneFeaturesByTissueWrapper <- function(sel.tissues.names, gene.features){
    all.genefeats.corr = matrix(0,nrow=length(sel.tissues.names), ncol=6)
    i = 1;
    for (t in sel.tissues.names){
        cat(t, sep="\n")
        res = plotGeneFeaturesByTissue(t, gene.features);
        all.genefeats.corr[i, 1] = as.character(t);
        all.genefeats.corr[i, 2] = res[[1]];
        all.genefeats.corr[i, 3] = res[[2]];
        all.genefeats.corr[i, 4] = res[[3]];
        all.genefeats.corr[i, 5] = res[[4]];
        all.genefeats.corr[i, 6] = res[[5]];
        i = i + 1;
    }
    write.table(all.genefeats.corr, "all.genefeats.corr.txt", quote=F, sep="\t",row.names=F)
}

# investigate if correlation with ischemic time is also related to expression levels
# for each tissue, take genes and create bins based on the respective correlation levels
# plot the expression distribution in each of the bins
# usage plotExpressionLevelByCorrBin(sel.sample.subj.covars, tissue.name, gene.rpkm)
plotExpressionLevelByCorrBin <- function(sample.matrix, tissue.name, rpkm){
    library(ggplot2)
    library(reshape)
    
    prefix.filename = "residualsregression/Regression.Residuals."
    tissue.names.nospaces = chartr(" ",".", tissue.name);   
    output_table_file.name = paste(prefix.filename, tissue.names.nospaces,".txt",sep="")
       
    # read the table with correlation info
    tissue.corr=read.table(output_table_file.name,h=T);
    colnames(tissue.corr) = c("index","genename","pearson.cor","pearson.pval","spearman.cor","spearman.pval");
    rownames(tissue.corr) = tissue.corr$genename;
    tissue.corr = tissue.corr[order(tissue.corr$pearson.cor),]
    
    # select respective RPKM values
    tissue.sample.ids = sample.matrix[sample.matrix$SMTSD==tissue.name,c("SAMPIDDOT")]
    sel.rpkm = gene.rpkm[rownames(gene.rpkm) %in% tissue.corr$genename,colnames(gene.rpkm) %in% tissue.sample.ids]
    
    # convert RPKM values into rank
    rank.rpkm = c()
    for(j in seq(1, ncol(sel.rpkm))){
        rank.rpkm = cbind(rank.rpkm, rank(sel.rpkm[,j]))
    }
    mean.rank = data.frame(apply(rank.rpkm,1,mean))
    rownames(mean.rank) = rownames(sel.rpkm)
    names(mean.rank) = "meanRank"
    m = merge(tissue.corr, mean.rank,by="row.names")
    #smoothScatter(m$pearson.cor, m$meanRank)
    return (cor(m$pearson.cor, m$meanRank)[1])
}

# Wrapper function for plotExpressionLevelByCorrBin
# generates a table that contains for each tissue the correlation of the correlation of gene expression and ischemic time with gene expression average rank
# across all samples of a given tissue
#usage: WrapperExpressionLevel(sel.sample.subj.covars, sel.tissues.names, gene.rpkm)
WrapperExpressionLevel <- function(matrix.vars, sel.tissues.names, rpkm){
    expression.corr = matrix(0,nrow=length(sel.tissues.names), ncol=2)
    i = 1;
    for (t in sel.tissues.names){
        r = plotExpressionLevelByCorrBin(matrix.vars, t, rpkm)
        expression.corr[i, 1] = t;
        expression.corr[i, 2] = r;
        cat(paste(t,r,sep=" "), sep="\n")
        i = i + 1
    }
    write.table(expression.corr, "correlation.geneexp_corr_ischemic.withExpressionRank.txt", quote=F, sep="\t",row.names=F)
}


########################################### ########################################### ########################################### 
            ########################################### ########################################### ########################################### 
########################################### ########################################### ########################################### 
# SPLICING ANALYSIS
########################################### ########################################### ###########################################
            ########################################### ########################################### ########################################### 
########################################### ########################################### ########################################### 
# PSI VALUES CALCULATED WITH IPSA PIPELINE
load("PSI/good_qual_exon_table_PSI_IPSA.bin")
ipsa.psi = good_qual_exon_table_PSI_IPSA
ipsa.psi = ipsa.psi * 100
rownames(ipsa.psi)=chartr("-",".",rownames(ipsa.psi))

# Auxiliary functions
corWithNA <- function(x){
    cor(x, method="pearson",use="pairwise.complete.obs")
}

#select exons with NAs in less than 10% of the cases
nonNAless <- function(x){
    if(sum(is.na(x))<round(length(x)*0.1)){
        TRUE
    } else {
        FALSE
    }
}

medianWithNA <- function(x){
   median(x, na.rm=TRUE) #convert the item with NA to median value from the column
}

meanWithNA <- function(x){
   median(x, na.rm=TRUE) #convert the item with NA to median value from the column
}

sdWithNA <- function(x){
   sd(x, na.rm=TRUE) #convert the item with NA to median value from the column
}

IQRwithNA <- function(x){
   IQR(x, na.rm=TRUE) #convert the item with NA to median value from the column
}

diffMinMax <- function(x){
    max(x, na.rm=TRUE) - min(x, na.rm=TRUE)
}

# test the correlation of each exon with the ischemic time (PMI)
# since we have a large number of exons, filter first by those exons that may have little variation
# correlation then PSI values with ischemic time
# corrPSI_ISCHEMICTIME(sel.sample.subj.covars, ipsa.psi, "IPSA")
corrPSI_ISCHEMICTIME <- function(sample.matrix, table.psi, name.psi){  
    for (tissue.name in sel.tissues.names){
        #select sample ids    
        tissue.sample.ids = sample.matrix[sample.matrix$SMTSD==tissue.name,c("SAMPIDDOT")]
        tissue.sample.ids = as.character(intersect(rownames(table.psi), as.character(tissue.sample.ids)))  
        sel.psi = t(table.psi[tissue.sample.ids ,])
        
        #select exons if few nonNAs
        sel.psi = sel.psi[apply(sel.psi,1,nonNAless),]
        # select exons if stdev is greater than 0, i.e. there is some variation in the PSI values
        sel.psi = sel.psi[apply(sel.psi,1,sdWithNA) > 0,]
        # select exons if difference in the max and min PSI values is larger than 10%
        sel.psi = sel.psi[apply(sel.psi,1,IQRwithNA)> 10,]
            
        #ischemic time of the samples
        sel.ischemictime = as.numeric(sample.matrix[as.character(colnames(sel.psi)),c("SMTSISCH")])
        
        #keep correlation values
        c = 1
        exon.corr <- matrix(nrow = nrow(sel.psi), ncol=4, 0);
        for (j in seq(1:nrow(sel.psi))){
                ct = cor.test(sel.ischemictime, as.numeric(sel.psi[j,]),use="na.or.complete");
                pval = as.numeric(as.vector(ct[3]$p.value));
                cat(paste(tissue.name, rownames(sel.psi)[j], ct$estimate[[1]], pval, sep="\t"),sep="\n")
                exon.corr[c,1] = rownames(sel.psi)[j]
                exon.corr[c,2] = ct$estimate[[1]]
                exon.corr[c,3] = pval
                exon.corr[c,4] = tissue.name
                c = c + 1
        }
        exoncorr.filename = paste("exon.corr.PSIwithISCHEMIC.", name.psi, ".", tissue.name, ".tab",sep="")
        write.table(exon.corr, exoncorr.filename ,quote=F,sep="\t")
    }
}

### Plot the correlation of an exon PSI value and the ischemic time of a sample
# plotExonCorr_PSI_ISCHEMIC(sel.sample.subj.covars, ipsa.psi, "Artery - Coronary", "chr21_40178375_40178732_+")

plotExonCorr_PSI_ISCHEMIC <- function(sample.matrix, table.psi, tissue.name, exon.name){
    tissue.color = as.character(tissue.colors.hash[as.character(tissue.name),c("color")])
    #select sample ids    
    tissue.sample.ids = sample.matrix[sample.matrix$SMTSD==tissue.name,c("SAMPIDDOT")]
    tissue.sample.ids = as.character(intersect(rownames(table.psi), as.character(tissue.sample.ids)))  
    sel.psi = t(table.psi[tissue.sample.ids ,])
    exon.psi = as.numeric(sel.psi[exon.name, ])
    
    #ischemic time of the samples
    sel.ischemictime = as.numeric(sample.matrix[as.character(colnames(sel.psi)),c("SMTSISCH")])
    rm(sel.psi)
    
    ct = cor.test(sel.ischemictime, exon.psi,use="na.or.complete");
    pval = as.numeric(as.vector(ct[3]$p.value));
    
    tit = paste(paste(exon.name, tissue.name,sep="\t"), paste("r=", ct$estimate[[1]], "p=", pval,sep=" "),sep="\n")
    # classic way to plot
    #plot(sel.ischemictime, exon.psi, xlab="PostMortem Interval(Minutes)", ylab="PSI",ylim=c(0,100),main=tit)
    dat = as.data.frame(cbind(sel.ischemictime, exon.psi))
    ## ggplot style
    out_filename = paste("ggp.exon.PMIvsPSI.", exon.name , ".", tissue.name , ".pdf",sep="")
    pdf(out_filename, w=11, h = 11, useDingbats=F)
        ggplot(dat, aes(x=sel.ischemictime, y=exon.psi)) + geom_point(size=3, shape=19, alpha=1/4,col="black") +  geom_smooth(method=lm) + theme(axis.text.y = element_text(colour = 'black', size = 12),axis.text.x = element_text(colour = 'black', size = 12)) + theme_bw() + labs(title = tit) + xlab("PMI") + ylab("PSI")
    dev.off()
}

#########################################################################################
# Analysis of recurrent exons; absolute r > 0.5 and FDR 1%
#########################################################################################
# call the above function to compute the correlation for IPSA, merge all tables in one
corrPSI_ISCHEMICTIME(sel.sample.subj.covars, ipsa.psi, "IPSA")
# in the command line merge all files
# cat exon.corr.PSIwithISCHEMIC.IPSA* > exon.corr.PSIwithISCHEMIC.IPSA.sel.tissues.tab
exon.corr.ipsa = read.table("exon.corr.PSIwithISCHEMIC.IPSA.sel.tissues.tab",h=T,sep="\t")
exon.corr.ipsa = exon.corr.ipsa[,-c(1)]
names(exon.corr.ipsa) = c("exon","corr","pval","tissue")
# make multiple test correction, by tissue
exon.corr.ipsa = cbind(exon.corr.ipsa, rep(0,nrow(exon.corr.ipsa)))
names(exon.corr.ipsa)[5] = "FDR"
for (tissue.name in sel.tissues.names){
    exon.corr.ipsa[exon.corr.ipsa$tissue==tissue.name,c("FDR")] = p.adjust(exon.corr.ipsa[exon.corr.ipsa$tissue==tissue.name,c("pval")],"BH")
}
write.table(exon.corr.ipsa, "exon.corr.PSIwithISCHEMIC.IPSA.sel.tissues.tab",quote=F,sep="\t")
# plot number of exons significantly associated with ischemic time per tissue, 
pdf("barplot.IPSA.exons.corr.ischemic.tissues.pdf", pointsize=16, w=14,h=12)recurrent.exon.genes.entrezids = bitr(recurrent.exon.genes, fromType="SYMBOL", toType="ENTREZID", annoDb="org.Hs.eg.db")
    par(mfrow=c(1,1), oma=c(12,0,0,0))
    exon.tab = table(exon.corr.ipsa[abs(exon.corr.ipsa$corr) > 0.5 & exon.corr.ipsa$FDR < 0.01,c("tissue")])
    exon.tab = exon.tab[order(exon.tab,decreasing=F)]
    tc = as.character(tissue.colors.hash[names(exon.tab),c(2)])
    barplot(exon.tab,las=2,ylim=c(0,250), main="#Exons significantly correlated \n(|r| > 0.5, FDR 1%) - IPSA PSI",ylab="Number of Exons",col = tc)
dev.off()

# GO analys for the most recurrently correlated exons across the different tissues
# table to associate gene names witexon ids
eg = read.table("/Gencode/gencode.exon.fullinfo.txt")
exons2genes = eg[,c(1,6,8)]
colnames(exons2genes) = c("exon","gene.id", "gene.name")
background.gene.list = unique(exons2genes$gene.name)
background.gene.list.entrezids = bitr(background.gene.list, fromType="SYMBOL", toType="ENTREZID", annoDb="org.Hs.eg.db")

###############################################
# Plot number of tissues where the exon is recurrent
sel.corr.exons = exon.corr.ipsa[abs(exon.corr.ipsa$corr) > 0.5 & exon.corr.ipsa$FDR <= 0.01,]
sel.corr.exons.rec.cnt = table(sel.corr.exons$exon)
# remove zero ones
sel.corr.exons.rec.cnt = sel.corr.exons.rec.cnt[sel.corr.exons.rec.cnt>0]
pdf("recurrentExons.countPerTissue.pdf",w=10,h=10,pointsize=16)
    hist(sel.corr.exons.rec.cnt,ylab="Number of Exons", xlab="Number of recurrent tissues",col="rosybrown4",xlim=c(1,15),main="")
dev.off()

# recurrence of exons in tissues with 20 or more significant exons
tissues.morethan20exons = table(exon.corr.ipsa[abs(exon.corr.ipsa$corr) > 0.5 & exon.corr.ipsa$FDR <= 0.01,c("tissue")])
names.tissues.morethan20exons = names(tissues.morethan20exons[tissues.morethan20exons>20])
shared.exons = matrix(nrow = 183, ncol=6, 0);
c=1
for (tissue.name.1 in names.tissues.morethan20exons){
    for (tissue.name.2 in names.tissues.morethan20exons){
        if(tissue.name.1 != tissue.name.2){
            exon.list.tissue.1 = exon.corr.ipsa[abs(exon.corr.ipsa$corr) > 0.5 & exon.corr.ipsa$FDR <= 0.01 & exon.corr.ipsa$tissue == tissue.name.1,c("exon")]
            exon.list.tissue.2 = exon.corr.ipsa[abs(exon.corr.ipsa$corr) > 0.5 & exon.corr.ipsa$FDR <= 0.01 & exon.corr.ipsa$tissue == tissue.name.2,c("exon")]
            isect = length(intersect(exon.list.tissue.1,exon.list.tissue.2))
            isect.ratio = isect / min(length(exon.list.tissue.1), length(exon.list.tissue.2))
            cat(paste(tissue.name.1,tissue.name.2,isect, isect.ratio,sep="\t"),sep="\n")
            shared.exons[c,1] = tissue.name.1
            shared.exons[c,2] = tissue.name.2
            shared.exons[c,3] = length(exon.list.tissue.1)
            shared.exons[c,4] = length(exon.list.tissue.2)
            shared.exons[c,5] = isect
            shared.exons[c,6] = isect.ratio
            c = c +1
        }
    }    
}
write.table(shared.exons, "tissue20ormoreexons.pairwise.exonsharing.txt",quote=F,row.names=F,sep="\t")
#################################################################################################

calculateGOEnrichmentRecurrentExons <- function(){
    library(reshape2)
    # count the recurrence of the exons in the different tissues
    exon.tab = table(exon.corr.ipsa[abs(exon.corr.ipsa$corr)>0.5 & exon.corr.ipsa$FDR <= 0.01,c(1)])
    exon.tab = exon.tab[exon.tab>=3]
    exon.tab = exon.tab[order(exon.tab)]
    exon.tab = as.matrix(exon.tab)
    exon.tab = as.data.frame(cbind(exon.tab,rownames(exon.tab)))
    names(exon.tab) = c("tissueCount","exon")
    exon.tab = merge(exon.tab, exons2genes, by.x="exon", by.y="exon")
    # genes with at least one exon correlated in at least 3 tissues
    recurrent.exon.genes = unique(exon.tab$gene.name)
    # Prepare gene lists for GO analysis
    # convert normal gene names for entrez IDS
    recurrent.exon.genes.entrezids = bitr(recurrent.exon.genes, fromType="SYMBOL", toType="ENTREZID", annoDb="org.Hs.eg.db")
    # function from Clusterprofiler for GO analysis
    # using the background  universe = background.gene.list.entrezids$ENTREZID
    kegg.recurrent.exons.genes <- enrichKEGG(gene = recurrent.exon.genes.entrezids$ENTREZID, universe = background.gene.list.entrezids$ENTREZID, organism="human", pvalueCutoff=0.01, use_internal_data = TRUE, qvalueCutoff=0.01, pAdjustMethod = "BH",readable=TRUE)
    # using the background  universe = background.gene.list.entrezids$ENTREZID
    go.CC.recurrent.exons.genes <- enrichGO(gene = recurrent.exon.genes.entrezids$ENTREZID, universe = background.gene.list.entrezids$ENTREZID , ont = "CC", organism="human", pvalueCutoff=0.01, qvalueCutoff=0.01, pAdjustMethod = "BH",readable=TRUE)
    go.MF.recurrent.exons.genes <- enrichGO(gene = recurrent.exon.genes.entrezids$ENTREZID, universe = background.gene.list.entrezids$ENTREZID , ont = "MF", organism="human", pvalueCutoff=0.01, qvalueCutoff=0.01, pAdjustMethod = "BH",readable=TRUE)
    go.BP.recurrent.exons.genes <- enrichGO(gene = recurrent.exon.genes.entrezids$ENTREZID,universe = background.gene.list.entrezids$ENTREZID , ont = "BP", organism="human", pvalueCutoff=0.01, qvalueCutoff=0.01, pAdjustMethod = "BH",readable=TRUE)

    # visualization
    pdf("barplot.go.MF.recurrent.exons.genes.pdf",w=10,h=10,pointsize=16)
        barplot(go.MF.recurrent.exons.genes,drop=TRUE, showCategory=4)
    dev.off()
        pdf("cnetplot.go.MF.recurrent.exons.genes.pdf",w=14,h=14,pointsize=16)
    cnetplot(go.MF.recurrent.exons.genes,categorySize="pvalue", showCategory=3)
    dev.off()
    pdf("cnetplot.kegg.recurrent.exons.genes.pdf",w=14,h=14,pointsize=16)
        cnetplot(kegg.recurrent.exons.genes,categorySize="geneNum")
    dev.off()

    #merge the results from both Go & KEGG
    kegg.top = head(summary(kegg.recurrent.exons.genes),5)[,c("Description","p.adjust")]
    cc.top = head(summary(go.CC.recurrent.exons.genes),5)[,c("Description","p.adjust")]
    bp.top = head(summary(go.BP.recurrent.exons.genes),5)[,c("Description","p.adjust")]
    mf.top = head(summary(go.MF.recurrent.exons.genes),5)[,c("Description","p.adjust")]
    go.kegg.desc.padj = rbind(cc.top, bp.top, mf.top, kegg.top)

    pdf("barplot.recurrentExons.IPSA.GO.KEGG.padj.pdf",w=14,h=14,pointsize=16)
        par(mfrow=c(1,1), oma=c(13,0,0,0))
        tc = c(rep("brown3",nrow(cc.top)), rep("cadetblue3",nrow(bp.top)), rep("darkolivegreen3",nrow(mf.top)), rep("burlywood1",nrow(kegg.top)))
        barplot(-log10(go.kegg.desc.padj$p.adjust),names=go.kegg.desc.padj$Description,las=2,ylab="-log10 FDR",ylim=c(0,12),main="Top 5 GO Categories & KEGG Pathways",col=tc, border=NA)
        legend("topright",  c("Cellular Component", "Biological Process","Molecular Functions","Pathways"), cex=1.2, col=c("brown3","cadetblue3","darkolivegreen3","burlywood1"), lwd=20,bty="n")
    dev.off()
}

#########################################################################
# Differential exon inclusion between Pre and Post-mortem Blood
# @ QUESTION : How many differentially included are found between Pre and Post mortem blood samples?
##########################################################################
diffInclusion.Blood.Pre.PostMortem <- function(table.psi){
    tissue.sample.ids = c(as.character(sel.blood.PREMORTEM),as.character(sel.blood.POSTMORTEM))
    tissue.sample.ids = as.character(intersect(rownames(table.psi), as.character(tissue.sample.ids)))  
    blood.pre.samples = as.character(intersect(rownames(table.psi), as.character(sel.blood.PREMORTEM)))
    blood.post.samples = as.character(intersect(rownames(table.psi), as.character(sel.blood.POSTMORTEM)))
    sel.psi = t(table.psi[tissue.sample.ids ,]);
     #select exons if few nonNAs
    sel.psi = sel.psi[apply(sel.psi,1,nonNAless),];
    # select exons if stdev is greater than 0, i.e. there is some variation in the PSI values
    sel.psi = sel.psi[apply(sel.psi,1,sdWithNA) > 0,]
    # select exons if difference in the max and min PSI values is larger than 10%
    sel.psi = sel.psi[apply(sel.psi,1,IQRwithNA)> 1,]
    c = 1
    blood.exon.diffinclusion <- matrix(nrow = nrow(sel.psi), ncol=4, 0);
    for (j in seq(1:nrow(sel.psi))){
            pre.exon.psi = as.numeric(sel.psi[c, blood.pre.samples]);
            post.exon.psi = as.numeric(sel.psi[c, blood.post.samples]);
            pre.mean = mean(pre.exon.psi, na.rm =T);
            post.mean = mean(post.exon.psi, na.rm =T);
            diff.mean = pre.mean - post.mean;
            wt = wilcox.test(pre.exon.psi, post.exon.psi,use="na.or.complete");
            pval = as.numeric(as.vector(wt[3]$p.value));
            #cat(paste(tissue.name, rownames(sel.psi)[j], wt$estimate[[1]], pval, sep="\t"),sep="\n")
            blood.exon.diffinclusion[c,1] = rownames(sel.psi)[j]
            blood.exon.diffinclusion[c,2] = as.numeric(wt[1]$statistic)
            blood.exon.diffinclusion[c,3] = pval
            blood.exon.diffinclusion[c,4] = diff.mean
            c = c + 1
    }
    exon.inclusion.filename = paste("exon.inclusion.", name.psi, ".Blood.Pre_Post", ".tab",sep="");
    write.table(blood.exon.diffinclusion, exon.inclusion.filename ,quote=F,sep="\t");
    blood.exon.diffinclusion = read.table(exon.inclusion.filename,h=T,sep="\t");
    names(blood.exon.diffinclusion) = c("exon","W","pval","diff");
    blood.exon.diffinclusion = cbind(blood.exon.diffinclusion, p.adjust(blood.exon.diffinclusion$pval,method="BH"));
    names(blood.exon.diffinclusion) = c("exon","W","pval", "diff","padj");
    blood.exon.diffinclusion = merge(blood.exon.diffinclusion, exons2genes, by.x="exon",by.y="exon");
    write.table(blood.exon.diffinclusion, exon.inclusion.filename ,quote=F,sep="\t");
    # number of exons with diff inclusion
    nexons.diffinc = nrow(blood.exon.diffinclusion[blood.exon.diffinclusion$padj< min_pval,]);
    #gene names for exons with differential inclusion in pre and post mortem blood samples
    write.table(blood.exon.diffinclusion[blood.exon.diffinclusion$padj< min_pval,c("gene.name")],"genenames.exons.diffInclusion.Pre_PostMortem.Blood.txt",quote=F,row.names=F)
    
    # exons differentially included in blood; |diff PSI| > 10% and pval < 0.01
    sel.blood.diffpsi = blood.exon.diffinclusion[blood.exon.diffinclusion$padj< 0.01 & abs(blood.exon.diffinclusion$diff)>10,]
    #genes from exons with higher inclusion in pre-mortem samples
    genes.exons.diffpsi.Included.Pre = unique(sel.blood.diffpsi[sel.blood.diffpsi$diff > 0,c("gene.name")])
    genes.exons.diffpsi.Included.Post = unique(sel.blood.diffpsi[sel.blood.diffpsi$diff < 0,c("gene.name")])
    write.table(sel.blood.diffpsi[sel.blood.diffpsi$diff > 0,], "/differential_PSI/genes.exons.diffpsi.Included.Pre.txt", quote=F, row.names=F, sep="\t")
    write.table(sel.blood.diffpsi[sel.blood.diffpsi$diff < 0,], "/differential_PSI/genes.exons.diffpsi.Included.Post.txt", quote=F, row.names=F, sep="\t")
}


#threshold = 0.05
#gene.list = genes.exons.diffpsi.Included.Pre
#gene.list = genes.exons.diffpsi.Included.Post
#enrichmentAnalysis_GO_KEGG_DifferentialBlood(gene.list, threshold)
enrichmentAnalysis_GO_KEGG_DifferentialBlood <- function(gene.list, threshold){
    library("DOSE")
    library("clusterProfiler")
    library(KEGG.db)
    library(pathview)

    # Universe of genes corresponds to all gene in the annotation file
    background.geneList = bitr(readcnt.blood[,c("Description")], fromType="SYMBOL", toType="ENTREZID", annoDb="org.Hs.eg.db")
    
    eg = bitr(gene.list, fromType="SYMBOL", toType="ENTREZID", annoDb="org.Hs.eg.db")
    gene <- eg$ENTREZID
    
    # get GO classification of all genes; may skip this
    go.BP.genes <- enrichGO(gene = eg$ENTREZID , ont = "BP", organism="human", pvalueCutoff=threshold, qvalueCutoff=threshold, pAdjustMethod = "BH",readable=TRUE)
    go.MF.genes <- enrichGO(gene = eg$ENTREZID , ont = "MF", organism="human", pvalueCutoff=threshold, qvalueCutoff=threshold, pAdjustMethod = "BH",readable=TRUE)
    go.CC.genes <- enrichGO(gene = eg$ENTREZID , ont = "CC", organism="human", pvalueCutoff=threshold, qvalueCutoff=threshold, pAdjustMethod = "BH",readable=TRUE)
    kegg <- enrichKEGG(gene = gene,organism = "human", pvalueCutoff = 0.01,readable= TRUE, use_internal_data = TRUE)
    #head(summary(kegg))
    #head(summary(ego.bp))
    pdf("KEGG.barplot.de_fc3.PremortemUP.pdf",w=18,h=14)
        barplot(kegg)
    dev.off()
    pdf("KEGG.de_fc3.PremortemUP.pdf",w=18,h=14)
        cnetplot(kegg)
    dev.off()

    # VISUALIZATION: other formats
    enrichMap(go.BP.genes)
    barplot(go.BP.genes, drop=TRUE, showCategory=20)
    cnetplot(go.BP.genes, categorySize="pvalue")

    #pdf("BP.enrichMap.diffpsi.Included.Pre.pdf")
    pdf("BP.enrichMap.diffpsi.Included.Post.pdf")
    enrichMap(go.BP.genes)
    dev.off()
    #pdf("MF.enrichMap.diffpsi.Included.Pre.pdf")
    pdf("MF.enrichMap.diffpsi.Included.Post.pdf")
    enrichMap(go.MF.genes)
    dev.off()
    #pdf("BP.barplot.diffpsi.Included.Pre.pdf")
    pdf("BP.barplot.diffpsi.Included.Pre.pdf")
    barplot(go.BP.genes, drop=TRUE, showCategory=20)
    dev.off()
    #pdf("MF.barplot.diffpsi.Included.Pre.pdf")
    pdf("MF.barplot.diffpsi.Included.Post.pdf")
    barplot(go.MF.genes, drop=TRUE, showCategory=20)
    dev.off()
    #pdf("KEGG.enrichMap.diffpsi.Included.Pre.pdf")
    pdf("KEGG.enrichMap.diffpsi.Included.Post.pdf")
    cnetplot(kegg)
    dev.off()
    
    write.table(summary(go.BP.genes),"/home/pferreira/Projects/GTEx2/ischemicTimeStudy/Blood_pre_post_comparison/differential_PSI/GO.BP.diffpsi.Included.Post.txt",quote=F,row.names=F,col.names=F,sep="\t")
    write.table(summary(go.MF.genes),"/home/pferreira/Projects/GTEx2/ischemicTimeStudy/Blood_pre_post_comparison/differential_PSI/GO.MF.diffpsi.Included.Post.txt",quote=F,row.names=F,col.names=F,sep="\t")
    write.table(summary(go.CC.genes),"/home/pferreira/Projects/GTEx2/ischemicTimeStudy/Blood_pre_post_comparison/differential_PSI/GO.CC.diffpsi.Included.Post.txt",quote=F,row.names=F,col.names=F,sep="\t")
    write.table(summary(kegg),"/home/pferreira/Projects/GTEx2/ischemicTimeStudy/Blood_pre_post_comparison/differential_PSI/KEGG.diffpsi.Included.Post.txt",quote=F,row.names=F,col.names=F,sep="\t")
    
    
    ## save files     
    #setwd("Blood_pre_post_comparison/GO/")
    #write.table(summary(go.BP.genes)[c(1,7)],"REVIGO.BP.de_fc3.PremortemUP.txt",quote=F,row.names=F,col.names=F,sep="\t")
    #write.table(summary(go.MF.genes)[c(1,7)],"REVIGO.MF.de_fc3.PremortemUP.txt",quote=F,row.names=F,col.names=F,sep="\t")
    #write.table(summary(go.CC.genes)[c(1,7)],"REVIGO.CC.de_fc3.PremortemUP.txt",quote=F,row.names=F,col.names=F,sep="\t")
    #write.table(summary(kegg),"KEGG.de_fc3.PremortemUP.txt",quote=F,row.names=F,col.names=F,sep="\t")

    gene.list.pre = genes.exons.diffpsi.Included.Pre
    gene.list.post = genes.exons.diffpsi.Included.Post
    eg = bitr(gene.list.pre, fromType="SYMBOL", toType="ENTREZID", annoDb="org.Hs.eg.db")
    gene.pre <- eg$ENTREZID
    eg = bitr(gene.list.post, fromType="SYMBOL", toType="ENTREZID", annoDb="org.Hs.eg.db")
    gene.post <- eg$ENTREZID
    gene.list.pre_pos = list(pre=gene.pre, post=gene.post)
    #paraqui
    #Enrichment function from Clusterprofiler for GO analysis
    kegg.pre.pos <- compareCluster(geneCluster = gene.list.pre_pos, fun = "enrichKEGG", organism="human", pvalueCutoff=threshold, use_internal_data = TRUE)
    go.pre.pos <- compareCluster(geneCluster = gene.list.pre_pos, fun = "enrichGO", organism="human", pvalueCutoff=threshold)
    
    # plot GO and KEGG Enrichment plots
    pdf("GO.pre_post.pdf",w=12,h=10)
        print(plot(go.pre.pos , type="dot",showCategory=3))
    dev.off()    
    pdf("KEGG.pre_post.pdf",w=6,h=6)
        print(plot(kegg.pre.pos, type="dot",showCategory=NULL))
    dev.off()
    
}

#############################################################################
# Exons differentially included between Pre and Post mortem blood samples?
#############################################################################
diffInclusion.Blood.groups <- function(table.psi, selected.exons, samples.groupA, samples.groupB, name.groupA, name.groupB){
    # ensure that all samples are in the psi table
    tissue.sample.ids = c(as.character(samples.groupA),as.character(samples.groupB))
    tissue.sample.ids = as.character(intersect(rownames(table.psi), as.character(tissue.sample.ids)))  
    blood.pre.samples = as.character(intersect(rownames(table.psi), as.character(samples.groupA)))
    blood.post.samples = as.character(intersect(rownames(table.psi), as.character(samples.groupB)))
    sel.psi = t(table.psi[tissue.sample.ids,]);
    #selected exons
    sel.psi = sel.psi[selected.exons,];
   
    c = 1
    blood.exon.diffinclusion <- matrix(nrow = nrow(sel.psi), ncol=4, 0);
    for (j in seq(1:nrow(sel.psi))){
            pre.exon.psi = as.numeric(sel.psi[c, blood.pre.samples]);
            post.exon.psi = as.numeric(sel.psi[c, blood.post.samples]);
            pre.mean = mean(pre.exon.psi, na.rm =T);
            post.mean = mean(post.exon.psi, na.rm =T);
            diff.mean = pre.mean - post.mean;
            wt = wilcox.test(pre.exon.psi, post.exon.psi,use="na.or.complete");
            pval = as.numeric(as.vector(wt[3]$p.value));
            cat(paste(rownames(sel.psi)[j], wt$estimate[[1]], pval, sep="\t"),sep="\n")
            blood.exon.diffinclusion[c,1] = rownames(sel.psi)[j]
            blood.exon.diffinclusion[c,2] = as.numeric(wt[1]$statistic)
            blood.exon.diffinclusion[c,3] = pval
            blood.exon.diffinclusion[c,4] = diff.mean
            c = c + 1
    }
    exon.inclusion.filename = paste("exon.inclusion.", name.groupA, "_vs_", name.groupB, ".tab",sep="");
    write.table(blood.exon.diffinclusion, exon.inclusion.filename ,quote=F,sep="\t",row.names=F);
    blood.exon.diffinclusion = read.table(exon.inclusion.filename,h=T,sep="\t");
    names(blood.exon.diffinclusion) = c("exon","W","pval","diff");
    blood.exon.diffinclusion = cbind(blood.exon.diffinclusion, p.adjust(blood.exon.diffinclusion$pval,method="BH"));
    names(blood.exon.diffinclusion) = c("exon","W","pval", "diff","padj");
    blood.exon.diffinclusion = merge(blood.exon.diffinclusion, exons2genes, by.x="exon",by.y="exon");
    write.table(blood.exon.diffinclusion, exon.inclusion.filename ,quote=F,sep="\t",row.names=F);
}

##############################################################################################
## SPLICING ENTROPY ALONG PMI
##############################################################################################
calculateTranscriptRatios <- function(){
    # transcript quantification
    flux.rpkm = read.table(gzfile("rna-seq/GTEx_Analysis_2015-01-12_RNA-seq_Flux1.6_transcript_rpkm.txt.gz"), header=T)
    gene.count = table(flux.rpkm$Gene_Symbol)
    # genes with 2 or more transcripts
    gene.names = names(gene.count[gene.count>1])
    # select transcripts per genes with 2 or more transcripts
    flux.rpkm.mt = flux.rpkm[flux.rpkm$Gene_Symbol %in% gene.names,]
    flux.rpkm.mt = flux.rpkm.mt[order(flux.rpkm.mt$Gene_Symbol),]
    
    flux.rpkm.ratios = flux.rpkm.mt[,c(1,2)]
    for (i in 5:length(names(flux.rpkm.mt))){
        idx = names(flux.rpkm.mt)[i]
        cat(idx, sep ="\n")
        perc <- ave(flux.rpkm.mt[,idx], list(flux.rpkm.mt$Gene_Symbol), FUN=function(L) L/sum(L))
        flux.rpkm.ratios = cbind(flux.rpkm.ratios, as.data.frame(perc))
    }
    names(flux.rpkm.ratios) = names(flux.rpkm.mt)[-c(3,4)]   
    rm(flux.rpkm)
    return (flux.rpkm.ratios)
}

# Correlate Splicing entropy with PMI
# usage : correlatePMIwithSplicingEntropy
correlatePMIwithSplicingEntropy <- function(sel.tissues.names, flux.rpkm.mt, flux.rpkm.ratios) {
    sel.tissues.names.nospaces = chartr(" ",".",sel.tissues.names)
    for (i in 1:length(sel.tissues.names)){
        tissue.name = sel.tissues.names[i]
        tissue.name.nospace = sel.tissues.names.nospaces[i]
        cat(tissue.name,sep="\n")
        selected.tissue.samples = sample.matrix[sample.matrix$SMTSD==tissue.name,c("SAMPIDDOT")]
        number.samples = length(selected.tissue.samples)
        sum.gene.exp = as.character(flux.rpkm.mt$Gene_Symbol)
        for (j in 1:length(selected.tissue.samples )){
            idx = selected.tissue.samples[j]
            tmp = flux.rpkm.mt[,c("Gene_Symbol", as.character(idx))]
            sum.gene.exp = cbind( sum.gene.exp ,ave(tmp[, as.character(idx)], list(tmp$Gene_Symbol), FUN=sum))
            cat(paste(j,idx,sep=" "), sep ="\n")
        }
        sum.gene.exp = as.data.frame(sum.gene.exp)
        names(sum.gene.exp) = c("Gene_Symbol", as.character(selected.tissue.samples))
        sum.gene.exp = unique(sum.gene.exp)
        write.table(sum.gene.exp, "sum.gene.exp.txt")
        sum.gene.exp.tab = read.table("sum.gene.exp.txt", h=T)
        # list of genes that have a non-zero expression in 90% of the samples
        genes.expressedAllsamples = sum.gene.exp.tab[rowSums(sum.gene.exp.tab[,-c(1)] > 0) > 0.9*number.samples,c(1)]
        #select the genes that have non-zero expression and the samples from the tissue
        selected.flux.ratios = flux.rpkm.ratios[flux.rpkm.ratios$Gene_Symbol %in% genes.expressedAllsamples, names(flux.rpkm.ratios) %in% c("TargetID","Gene_Symbol", as.character(selected.tissue.samples))]
        rm(sum.gene.exp)
        #calculate the entropy SUM pi * log pi for each gene        
        entropy.gene.table =  as.character(selected.flux.ratios[,c("Gene_Symbol")])
        for (j in 1:length(selected.tissue.samples)){
            idx = selected.tissue.samples[j]
            tmp = selected.flux.ratios[,c("Gene_Symbol", as.character(idx))]
            entropy.gene.table = cbind(entropy.gene.table ,ave(tmp[,as.character(idx)], list(tmp$Gene_Symbol), FUN=function(L){-1*sum((L+0.0001)*log(L+0.0001))}))
            cat(paste(j,idx,sep=" "), sep ="\n")
        }
        entropy.gene.table = as.data.frame(entropy.gene.table)
        names(entropy.gene.table) = c("Gene_Symbol", as.character(selected.tissue.samples))
        entropy.gene.table = unique(entropy.gene.table)
        write.table(entropy.gene.table , "entropy.gene.table.txt")
        entropy.gene.table = read.table("entropy.gene.table.txt", h=T)
        rownames(entropy.gene.table) = entropy.gene.table$Gene_Symbol
        # now correlate the splicing entropy with the ischemic time for each gene
        ischemic.time = sel.sample_attribs_subj[sel.sample_attribs_subj$SAMPIDDOT %in% selected.tissue.samples,c("SAMPIDDOT", "SMTSISCH")]
        ischemic.time = ischemic.time[order(ischemic.time$SMTSISCH),]
        entropy.gene.table = entropy.gene.table[,as.character(ischemic.time$SAMPIDDOT)]
        
        entropy.cor.mat = matrix(nrow=nrow(entropy.gene.table),ncol=6);
        for(h in 1:nrow(entropy.gene.table)){
            gname = as.character(rownames(entropy.gene.table)[i])
            entropy = as.numeric(entropy.gene.table[i,])
            # test correlation
            cc.pearson = cor.test(entropy,ischemic.time$SMTSISCH,method="pearson",use="na.or.complete")
            cc.spearman = cor.test(entropy,ischemic.time$SMTSISCH,method="spearman",use="na.or.complete")
            cc.pearson.cor = as.numeric(cc.pearson$estimate)
            cc.pearson.pval = as.numeric(cc.pearson$p.value)
            cc.spearman.cor = as.numeric(cc.spearman$estimate)
            cc.spearman.pval = as.numeric(cc.spearman$p.value)
            
            cat(paste(gname, cc.pearson.cor,sep=" "), sep="\n")
            entropy.cor.mat[j,1]=i
            entropy.cor.mat[j,2]=as.character(gname)
            entropy.cor.mat[j,3]=cc.pearson.cor
            entropy.cor.mat[j,4]=cc.pearson.pval
            entropy.cor.mat[j,5]=cc.spearman.cor
            entropy.cor.mat[j,6]=cc.spearman.pval
        }
        entropy.cor.mat = as.data.frame(entropy.cor.mat)
        names(entropy.cor.mat) = c("idx", "genename", "pearson.cor", "pearson.pval", "spearman.cor","spearman.pval")
        entropy.cor.mat = merge(entropy.cor.mat, gene2name, by.x="genename", by.y="Name")
        output_table_file.name = paste("IsoformEntropy/Splicing.Entropy.", tissue.name.nospace,".txt",sep="")
        write.table(entropy.cor.mat, output_table_file.name, quote=F, row.names=F,sep="\t")
        entropy.cor.mat = read.table(output_table_file.name, h=T)
    }       
}

# create a visualization for a given gene of the PMI vs Entropy
plotSplicingEntropyGeneTissue <- function(tissue.name, gid, flux.rpkm.ratios){
    gname=as.character(gene2name[gene2name$Name==gid,c("Description")])
    # ischemic time of the samples of the given tissues
    selected.tissue.samples = sample.matrix[sample.matrix$SMTSD==tissue.name,c("SAMPIDDOT")]
    ischemic.time = sel.sample_attribs_subj[sel.sample_attribs_subj$SAMPIDDOT %in% selected.tissue.samples,c("SAMPIDDOT", "SMTSISCH")]
    ischemic.time = ischemic.time[order(ischemic.time$SMTSISCH, decreasing=T),]
    #create a color gradient according to the ischemic time of the sample
    isch.col = rep("gray",nrow(ischemic.time))
    colfunc <- colorRampPalette(c("green", "yellow", "red"))
    isch.gradient = colfunc(max(ischemic.time$SMTSISCH,na.rm=T)+1)
    for (i in seq(1:length(ischemic.time$SMTSISCH))){
        it = ischemic.time$SMTSISCH[i]
        if(is.na(it)) {
            isch.col[i] = "#778899"
        }else{
            if(it < 0) {isch.col[i] = "#FFFF00"}
            if(it >= 0) {it = it + 1; isch.col[i] = isch.gradient[it]}
        }   
    }
    # get the transcript ratios for the given gene
    trx.ids = flux.rpkm.ratios[flux.rpkm.ratios$Gene_Symbol==gid,c("TargetID")] 
    transcripts.ratios.2plot = t(as.matrix(flux.rpkm.ratios[flux.rpkm.ratios$Gene_Symbol==gid,as.character(ischemic.time$SAMPIDDOT)]))
    marker = list(color = colorRampPalette(brewer.pal(9,"OrRd"))(100))
    heatmap.2(transcripts.ratios.2plot,Rowv=NA,Colv=NA,col=marker$color,scale="none",dendrogram="none",trace="none",labCol=trx.ids,RowSideColors=isch.col,keysize=1.0,symkey=F,main=gname,labRow=ischemic.time$SMTSISCH, denscol=F)
}

# plot entropy for gene APBB1IP in Lung
gid="ENSG00000067225.13"
pdf("APBB1IP_Lung.pdf",w=7,h=16)
    par(oma=c(16,0,0,0))
    plotSplicingEntropyGeneTissue("Lung",gid,flux.rpkm.ratios)
dev.off()


# statistics on the number of genes with splicing entropy correlated with PMI
stat.splicEnt = statsSplicingEntropyPMIperTissue(sel.tissues.names, 0.5, 0.05)

statsSplicingEntropyPMIperTissue <- function(sel.tissues.names, min.out.corr.val, min.adj.pval) {
    splicingEnt.perTissue <- matrix(nrow = length(sel.tissues.names) , ncol=3, 0);
    sel.tissues.names.nospaces = chartr(" ",".",sel.tissues.names)
    for (i in 1:length(sel.tissues.names)){
        tissue.name = sel.tissues.names[i]
        tissue.name.nospace = sel.tissues.names.nospaces[i]
        output_table_file.name = paste("IsoformEntropy/Splicing.Entropy.",sel.tissues.names.nospaces[i],".txt",sep="")
        cat(output_table_file.name ,sep="\n")
        entropy.cor.mat = read.table(output_table_file.name, h=T)
        entropy.cor.mat = cbind(entropy.cor.mat,p.adjust(entropy.cor.mat$pearson.pval,method="BH"))
        names(entropy.cor.mat)[8] = "padj"
        signif.genes.splic =  nrow(entropy.cor.mat[abs(entropy.cor.mat$pearson.cor)>0.5 & entropy.cor.mat$padj<0.05 & !is.na(entropy.cor.mat$pearson.cor),])
        splicingEnt.perTissue[i,1] = as.character(tissue.name)
        splicingEnt.perTissue[i,2] = nrow(entropy.cor.mat)
        splicingEnt.perTissue[i,3] = signif.genes.splic
    }
        splicingEnt.perTissue = data.frame(splicingEnt.perTissue)
        write.table(splicingEnt.perTissue, "splicingEntropy.perTissue.stats.txt",sep="\t",row.names=F,quote=F)
        splicingEnt.perTissue = read.table("splicingEntropy.perTissue.stats.txt", h=T)
        return (splicingEnt.perTissue)
}

 #####################################################################################################################
#########                           TIME COURSE DIFFERENTIAL EXPRESSION ANALYSIS          ####################################
    #####################################################################################################################


###################################################################################
# For a given tissue prepare gene expression table per tissue
###################################################################################
# usage: exptab = prepareTissueExpressionTable("Kidney - Cortex")
prepareTissueExpressionTable <- function(tissue.name, user.sel.genes = NULL){
    library(preprocessCore)
    selected.tissue.samples = sample.matrix[sample.matrix$SMTSD==tissue.name,c("SAMPIDDOT")]
    # selects the gene expression for the input samples, converts to log2 scale and quantile normalizes the expression log2 transformed values
    cnames = colnames(gene.rpkm[,colnames(gene.rpkm) %in% selected.tissue.samples])
    rpkm.tissue = gene.rpkm[,colnames(gene.rpkm) %in% selected.tissue.samples]
    rpkm.tissue = log2(1+rpkm.tissue);
    rpkm.tissue = as.data.frame(normalize.quantiles(as.matrix(rpkm.tissue)));
    names(rpkm.tissue) = cnames
    rownames(rpkm.tissue) = rownames(gene.rpkm)
    
    # get ischemic time per sample and order samples by ischemic time
    sel.ischemictime = sample.matrix[as.character(selected.tissue.samples),c("SAMPIDDOT","SMTSISCH")]
    sel.ischemictime = sel.ischemictime[order(sel.ischemictime$SMTSISCH),]
    
    # select samples with positive ischemic times only (remove pre-mortem samples)
    sel.ischemictime = sel.ischemictime[sel.ischemictime$SMTSISCH >0 & !is.na(sel.ischemictime$SMTSISCH),]
    
    # order columns by sample ischemic time
    rpkm.tissue = rpkm.tissue[,as.character(sel.ischemictime$SAMPIDDOT)]
    
    if (is.null(user.sel.genes)){
        # remove genes where sum of RPKMs is minimal
        sel.genes = names(which(rowSums(rpkm.tissue > 0.5) > 0.5 * ncol(rpkm.tissue)))
    }else{
        sel.genes = user.sel.genes
    }
    rpkm.tissue = rpkm.tissue[sel.genes,]
    tsd = apply(rpkm.tissue,1,sd)
    tsd = tsd[order(tsd)]
    return (rpkm.tissue)   
}

###################################################################################
# For a given tissue prepare a list of ischemic time values for the samples on that tissue
###################################################################################
getTissueIschemicTime <-function(tissue.name){
    selected.tissue.samples = sample.matrix[sample.matrix$SMTSD==tissue.name,c("SAMPIDDOT")]
    # get ischemic time per sample and order samples by ischemic
    sel.ischemictime = sample.matrix[as.character(selected.tissue.samples),c("SAMPIDDOT","SMTSISCH")]
    sel.ischemictime = sel.ischemictime[order(sel.ischemictime$SMTSISCH),]
    # select samples with positive ischemic times only (remove pre-mortem samples)
    sel.ischemictime = sel.ischemictime[sel.ischemictime$SMTSISCH >0,]
    return (sel.ischemictime)
}

# plot the expression of a gene with relation to ischemic time
# it receives a table with expression values and the ischemic time values and the gene id
# usage: plotGeneExpressionProfile(tissue.rpkm, tissue.isch, "ENSG00000211459.2")
plotGeneExpressionProfile <- function(expTab, tissue.isch, gname){     
        tissue.expr = t(exptab)
        gene.list = colnames(exptab)[moduleColors.tissue=="pink"]
        sizeGrWindow(15, 15)
        par(mfrow = c(5,5));
        for(i in seq(1:25)){
            gname = gene.list[i]
            cat(gname,sep="\n")
            gexp = as.numeric(tissue.expr[grep(gname,rownames(tissue.expr)),])
            gl.spline = smooth.spline(tissue.isch$SMTSISCH, gexp)
            plot(tissue.isch$SMTSISCH, gexp,log="x")
            lines(gl.spline, col = "red")
        }
}


######################################################################################################## the detectTimeIntervalChanges function is an ad hoc function that detect changes between two consecutive time intervals
# to calculate the change between time intervals: i) calculate the median
# ii) calculate the wilcoxon text between every two pair of consecutive time points
# if fold change is greater than 2 abs(log2-fc) > 1 and pval > min_pval then the gene is significant
# report each significant gene 
######################################################################################################
detectTimeIntervalChanges <- function(tissue.name, PVAL){
    tissue.expr = prepareTissueExpressionTable(tissue.name, NULL)
    tissue.isch = getTissueIschemicTime(tissue.name)
    tissue.name.nospace = chartr(" ",".",tissue.name)
    tissue.expr = tissue.expr[,!(colnames(tissue.expr) %in% as.character(tissue.isch[is.na(tissue.isch$SMTSISCH),c("SAMPIDDOT")]))]
    tissue.isch = tissue.isch[!(tissue.isch$SAMPIDDOT %in% as.character(tissue.isch[is.na(tissue.isch$SMTSISCH),c("SAMPIDDOT")])),]
   
    gene.signif.list = c()
    
    out_filename = paste("plot.Time_",tissue.name.nospace, ".pdf",sep="")
    pdf(out_filename, w=15, h=15)
    par(mfrow = c(5,5));
    
    for(i in seq(1:nrow(tissue.expr))){
        gname = as.character(rownames(tissue.expr)[i])
        chrom = as.character(chroms[gname,c("chrom")])
        gexp = as.numeric(tissue.expr[grep(gname,rownames(tissue.expr)),])
        # Z score normalize
        z = (gexp-mean(gexp))/sd(gexp)
        pp = cbind(tissue.isch, gexp)
        # exclude genes from sex chromossomes
        if ((chrom != "chrX") & (chrom != "chrY")){
            # divide in five temporal bins
            a1=pp[pp$SMTSISCH>=0 & pp$SMTSISCH<60,c("gexp")]
            a2=pp[pp$SMTSISCH>=60 & pp$SMTSISCH<240,c("gexp")]  
            a3=pp[pp$SMTSISCH>=240 & pp$SMTSISCH<360,c("gexp")] 
            a4=pp[pp$SMTSISCH>=360 & pp$SMTSISCH<900,c("gexp")]   
            a5=pp[pp$SMTSISCH>=900,c("gexp")]
            
            if (length(a1)==0) {a1=c(0)}
            if (length(a2)==0) {a2=c(0)}
            if (length(a3)==0) {a3=c(0)}
            if (length(a4)==0) {a4=c(0)}
            if (length(a5)==0) {a5=c(0)}
                   
            a1_med = median(a1) + 0.01
            a2_med = median(a2) + 0.01
            a3_med = median(a3) + 0.01
            a4_med = median(a4) + 0.01
            a5_med = median(a5) + 0.01
            # require at least 5 samples per bin for testing
            if((length(a1)>5) & (length(a2)>5)){
                wt_a1_a2 = wilcox.test(a1, a2);
                pval_a1_a2 = wt_a1_a2$p.value;
                fc_a1_a2 = log2(a1_med/a2_med);
            }else{
                pval_a1_a2 = 1;
                fc_a1_a2 = 0;
            }
            if((length(a2)>5) & (length(a3)>5)){
                wt_a2_a3 = wilcox.test(a2, a3);
                pval_a2_a3 = wt_a2_a3$p.value;
                fc_a2_a3 = log2(a2_med/a3_med);
            }else{
                pval_a2_a3 = 1;
                fc_a2_a3 = 0;
            }
            if((length(a3)>5) & (length(a4)>5)){
                wt_a3_a4 = wilcox.test(a3, a4);
                pval_a3_a4 = wt_a3_a4$p.value;
                fc_a3_a4 = log2(a3_med/a4_med);
            }else{
                pval_a3_a4 = 1;
                fc_a3_a4 = 0;
            }
            if((length(a4)>5) & (length(a5)>5)){
                wt_a4_a5 = wilcox.test(a4, a5);
                pval_a4_a5 = wt_a4_a5$p.value;
                fc_a4_a5 = log2(a4_med/a5_med);
            }else{
                pval_a4_a5 = 1;
                fc_a4_a5 = 0;
            }
            # get common gene names
            common.genename = as.character(gene2name[gname,c("Description")])
            # plot 5 bins orange indicates significant change between intervals
            if((pval_a1_a2 < PVAL) & (abs(fc_a1_a2)> 1) & (length(a1)>5) & (length(a2)>5)){
                boxplot(a1,a2,a3,a4,a5,main=gene2name[gname,c("Description")],col=c("orange","orange","gray","gray","gray"))
                cat(common.genename,sep="\n")
                if(fc_a1_a2 > 0){
                    gene.signif.list = rbind(gene.signif.list, c(gname,"1-2", "Inc"))
                }else{
                    gene.signif.list = rbind(gene.signif.list, c(gname,"1-2", "Dec"))
                }
            }
            if((pval_a2_a3 < PVAL) & (abs(fc_a2_a3)> 1) & (length(a2)>5) & (length(a3)>5)){
                boxplot(a1,a2,a3,a4,a5,main=gene2name[gname,c("Description")],col=c("gray","orange","orange","gray","gray"))
                cat(common.genename,sep="\n")
                if(fc_a2_a3 > 0){
                    gene.signif.list = rbind(gene.signif.list, c(gname,"2-3", "Inc"))
                }else{
                    gene.signif.list = rbind(gene.signif.list, c(gname,"2-3", "Dec"))
                }
            }
            if((pval_a3_a4 < PVAL) & (abs(fc_a3_a4)> 1) & (length(a3)>5) & (length(a4)>5)){
                boxplot(a1,a2,a3,a4,a5,main=gene2name[gname,c("Description")],col=c("gray","gray","orange","orange","gray"))
                cat(common.genename,sep="\n")
                if(fc_a3_a4 > 0){
                    gene.signif.list = rbind(gene.signif.list, c(gname,"3-4", "Inc"))
                }else{
                    gene.signif.list = rbind(gene.signif.list, c(gname,"3-4", "Dec"))
                }
            }
            if((pval_a4_a5 < PVAL) & (abs(fc_a4_a5)> 1) & (length(a4)>5) & (length(a5)>5)){
                boxplot(a1,a2,a3,a4,a5,main=gene2name[gname,c("Description")],col=c("gray","gray","gray","orange","orange"))
                cat(common.genename,sep="\n")
                if(fc_a4_a5 > 0){
                    gene.signif.list = rbind(gene.signif.list, c(gname,"4-5", "Inc"))
                }else{
                    gene.signif.list = rbind(gene.signif.list, c(gname,"4-5", "Dec"))
                }
            }
        }
    }
    dev.off()
    gene.signif.list = as.data.frame(gene.signif.list)
    colnames(gene.signif.list)=c("gene","interval", "direction")
    return(gene.signif.list)
}

# Get the number of genes tested by issues
detectTimeIntervalChangesNumberTestedGenes <- function(){
    temp.testedgenes <- matrix(nrow = 36, ncol=2, "");
    i = 1
    for(tissue.name in sel.tissues.names){
        tissue.expr = prepareTissueExpressionTable(tissue.name, NULL)
        tissue.name.nospace = chartr(" ",".",tissue.name)
        temp.testedgenes[i, 1] = tissue.name
        temp.testedgenes[i, 2] = nrow(tissue.expr)
        cat(paste(tissue.name.nospace, nrow(tissue.expr)), sep="\n")
        i = i + 1
    }
    setwd("/IntervalGeneExpressionChange")
    write.table(temp.testedgenes, "number.testedGenes.pertissue.txt", quote=F, sep="\t",row.names=F)
}


#############################################################################################
# Compiles the list of recurrent genes across num.recurrent.tissues with changes between time intervals
#############################################################################################
compileGeneListWithIntervalChange <- function(num.recurrent.tissues){
    setwd("/IntervalGeneExpressionChange/")
    library(reshape2)
    
    gtc=read.table("allgenes.tissue.change.txt",h=F,sep="\t")
    names(gtc)=c("gene","tissue","interval")
    gtc.gene = table(gtc$gene)
    gtc.gene = gtc.gene[order(gtc.gene)]
    gtc.gene = cbind(names(gtc.gene),as.numeric(gtc.gene))
    gtc.gene = as.data.frame(gtc.gene)
    names(gtc.gene) = c("gene","count")
    gtc = merge(gtc,gtc.gene,by.x="gene",by.y="gene")
    gtc = as.data.frame(gtc)
    write.table(gtc,"gtc.txt", sep="\t")
    gtc = read.table("gtc.txt",h=T,sep="\t")
    
    gtc.gt = gtc[gtc$count >= num.recurrent.tissues,]
    gtc.gt.ids = as.data.frame(unique(gtc[gtc$count >= num.recurrent.tissues,c("gene")]))
    names(gtc.gt.ids)=c("id")
    return (gtc.gt)    
}

#############################################################################################
# plot the list of recurrent genes across the different tissues and the respective time interval where changes were detected
#############################################################################################
plotHeatmapRecurrentGeneListWithIntervalChange <- function(num.recurrent.tissues){
    gtc.gt.numrec = compileGeneListWithIntervalChange(num.recurrent.tissues)
    out_filename = paste("Heatmap.RecurrentGenes.IschemicTimeIntervalsChanges_", num.recurrent.tissues , ".pdf",sep="")
    pdf(out_filename,w=20,h=20)
        ggplot(gtc.gt.numrec, aes(x=tissue, y=gene, fill=interval)) + geom_tile(aes(height=4))  +   scale_fill_manual(values=c("green","greenyellow","yellow","orange","red")) + theme(axis.text.x =  element_text(colour = 'black', angle = 90, size = 12)) + theme(axis.text.y = element_text(colour = 'black', size = 9)) + scale_y_discrete(limits=(gtc.gt2$gene)[order(gtc.gt2$count)])
    dev.off()
}


# Given a list of gene ids retrieves KEGG and GO terms
# call the function enrichedTermsGoKegg(sel.genes.ids, 0.05)
calculateGeneEnrichment <- function(num.recurrent.tissues){
    gtc.gt.numrec = compileGeneListWithIntervalChange(num.recurrent.tissues)
    sel.genes.ids = unique(gtc.gt.numrec$gene)
    gene.enrich = enrichedTermsGoKegg(sel.genes.ids, 0.05)
    write.table(summary(gene.enrich$cc),"GO.CC.recurrentGenes.gt3tissues.txt", sep="\t")
    write.table(summary(gene.enrich$mf),"GO.MF.recurrentGenes.gt3tissues.txt", sep="\t")
    write.table(summary(gene.enrich$bp),"GO.BP.recurrentGenes.gt3tissues.txt", sep="\t")
    write.table(summary( kegg.genes),"KEGG.recurrentGenes.gt3tissues.txt", sep="\t")
    return (gene.enrich)
}


enrichedTermsGoKegg <- function(sel.genes.ids, threshold){
    library("DOSE")
    library("clusterProfiler")
    library(KEGG.db)
    # convert ids
    eg = bitr(sel.genes.ids, fromType="SYMBOL", toType="ENTREZID", annoDb="org.Hs.eg.db")
    kegg.genes <- enrichKEGG(gene = eg$ENTREZID, organism="human", pvalueCutoff=threshold, use_internal_data = TRUE, qvalueCutoff=threshold, pAdjustMethod = "BH",readable=TRUE)
    go.BP.genes <- enrichGO(gene = eg$ENTREZID , ont = "BP", organism="human", pvalueCutoff=threshold, qvalueCutoff=threshold, pAdjustMethod = "BH",readable=TRUE)
    go.MF.genes <- enrichGO(gene = eg$ENTREZID , ont = "MF", organism="human", pvalueCutoff=threshold, qvalueCutoff=threshold, pAdjustMethod = "BH",readable=TRUE)
    go.CC.genes <- enrichGO(gene = eg$ENTREZID , ont = "CC", organism="human", pvalueCutoff=threshold, qvalueCutoff=threshold, pAdjustMethod = "BH",readable=TRUE)
    
    gene.enrich = list(kegg=kegg.genes, bp=go.BP.genes , mf=go.MF.genes, cc=go.CC.genes)
    return (gene.enrich)
}



###############################################################################################
#plot for the distribution of gene expression in time intervals across different tissue
###############################################################################################
# to select the genes and the tissues of interest in the command line
# grep HBA1 gene.tissue.change.txt | cut -f2 | awk 'BEGIN{ORS=",";print "tissues.of.interest = c("}{print "\""$1"\""}END{print ")"}'
        
#tissues.of.interest = c("Adipose - Visceral (Omentum)","Artery - Aorta","Colon - Sigmoid","Colon - Transverse","Esophagus - Gastroesophageal Junction","Heart - Atrial Appendage","Heart - Left Ventricle","Minor Salivary Gland","Ovary","Prostate","Stomach","Uterus","Vagina")
# call the function plotTissuesTimeIntervalGeneExpression
# plotTissuesTimeIntervalGeneExpression("HBA1", tissues.of.interest)

# plot the genes for the different input tissues in the time intervals with stdev values
plotTissuesTimeIntervalGeneExpression <- function(gene.name, tissues.of.interest){
    gene.id = as.character(gene2name[gene2name$Description==gene.name,c("Name")])
   
    gene.byTissue.byInterval.dist = c()
    for(tissue.name in tissues.of.interest){
        cat(tissue.name,sep="\n")
        # get expression values of the gene
        tissue.isch = getTissueIschemicTime(tissue.name)
        tissue.expr = prepareTissueExpressionTable(tissue.name, NULL)
        gene.expr = tissue.expr[gene.id,as.character(tissue.isch$SAMPIDDOT)]
        tissue.name.nospace = chartr(" ",".",tissue.name)
        # gene and tissue values
        gene.expr = gene.expr[,!(colnames(gene.expr) %in% as.character(tissue.isch[is.na(tissue.isch$SMTSISCH),c("SAMPIDDOT")]))]
        tissue.isch = tissue.isch[!(tissue.isch$SAMPIDDOT %in% as.character(tissue.isch[is.na(tissue.isch$SMTSISCH),c("SAMPIDDOT")])),]
        pp = c()
        gexp = as.numeric(gene.expr)
        gexp = (gexp-mean(gexp))/sd(gexp)
        
        pp = cbind(tissue.isch, gexp)
        a1=pp[pp$SMTSISCH<60,c("gexp")]
        a2=pp[pp$SMTSISCH>=60 & pp$SMTSISCH<240,c("gexp")]  
        a3=pp[pp$SMTSISCH>=240 & pp$SMTSISCH<360,c("gexp")] 
        a4=pp[pp$SMTSISCH>=360 & pp$SMTSISCH<900,c("gexp")]   
        a5=pp[pp$SMTSISCH>=900,c("gexp")]
        
        gene.byTissue.byInterval.dist = rbind(gene.byTissue.byInterval.dist,c(tissue.name, "I1", mean(a1),sd(a1)))
        gene.byTissue.byInterval.dist = rbind(gene.byTissue.byInterval.dist,c(tissue.name, "I2", mean(a2),sd(a2)))
        gene.byTissue.byInterval.dist = rbind(gene.byTissue.byInterval.dist,c(tissue.name, "I3", mean(a3),sd(a3)))
        gene.byTissue.byInterval.dist = rbind(gene.byTissue.byInterval.dist,c(tissue.name, "I4", mean(a4),sd(a4)))
        gene.byTissue.byInterval.dist = rbind(gene.byTissue.byInterval.dist,c(tissue.name, "I5", mean(a5),sd(a5)))
    }
        write.table(gene.byTissue.byInterval.dist, "gene.byTissue.byInterval.dist.txt", quote=F, sep="\t", row.names=F)
        gene.byTissue.byInterval.dist = read.table("gene.byTissue.byInterval.dist.txt",h=T, sep="\t")
        names(gene.byTissue.byInterval.dist) = c("Tissue","Interval","Mean","Stdev")
        
        gene.byTissue.byInterval.dist = gene.byTissue.byInterval.dist[gene.byTissue.byInterval.dist$Tissue %in% tissues.of.interest,]
        gene.byTissue.byInterval.dist = gene.byTissue.byInterval.dist[order(gene.byTissue.byInterval.dist$Tissue),]

        out_filename = paste("temporalInterv.stdev.",gene.name, ".pdf",sep="")
        
        pdf(out_filename, w=14,h=12)
            ggplot(gene.byTissue.byInterval.dist, aes(x=Interval, y=Mean, group=Tissue, color=Tissue)) + geom_errorbar(aes(ymin=Mean-Stdev, ymax=Mean+Stdev), size=1.0, position=position_dodge(0.07)) + geom_line(size=2) + geom_point(size=4, shape=21, fill="white") + theme(legend.background = element_rect(colour = "black"),legend.text=element_text(size=14),panel.background = element_rect(colour = "gray64"), axis.title=element_text(size=14), axis.text=element_text(size=14), strip.text.y = element_text(size = 11, hjust = 0.5, vjust = 0.5, face = 'bold'),  panel.margin = unit(0.25, "cm")) + scale_color_manual(values=as.character(tissue.colors.hash[as.character(unique(gene.byTissue.byInterval.dist$Tissue)),c("color")])) + ggtitle(gene.name)
        dev.off()
}


################################################################################################
# This function plots heatmaps for genes with significant change from time interval to another time interval; the heatmap contains the expression of those genes scaled by row; the columns are ordered by increasing ischemic time.
# it works in two modes: mode = 1 plots a more simpler heatmap;
# mode = 2 plots heatmap with color code for GO annotation 
################################################################################################
# usage: createHeatmapSelGenesIschemicTime(tissue.name, genes.change.list)
createHeatmapSelGenesIschemicTime <- function(tissue.name, sel.genes.ids, minNumGenes, mode){
    library(gplots)
    library(RColorBrewer)
    sel.genes.ids = as.character(unique(sel.genes.ids))
     
    tissue.expr = prepareTissueExpressionTable(tissue.name, NULL)
    tissue.isch = getTissueIschemicTime(tissue.name)
    tissue.name.nospace = chartr(" ",".",tissue.name)
    
    tissue.expr=tissue.expr[,!(colnames(tissue.expr) %in% as.character(tissue.isch[is.na(tissue.isch$SMTSISCH),c("SAMPIDDOT")]))]
    tissue.isch=tissue.isch[!(tissue.isch$SAMPIDDOT %in% as.character(tissue.isch[is.na(tissue.isch$SMTSISCH),c("SAMPIDDOT")])),]
    
    # create a color division according to the ischemic time intervals as defined below:
    # 0-1hr; 1-4hr; 4-6hr; 6-15 hr; +15hr
    colfunc <- colorRampPalette(c("green", "yellow", "red"))
    isch.gradient =  colfunc(max(tissue.isch$SMTSISCH)  + 1)
    isch.col = rep("gray",length(colnames(tissue.expr)))
    
    for (i in seq(1:length(isch.col))){
        it = tissue.isch[i,c("SMTSISCH")]
        if(is.na(it)) {
            isch.col[i] = "#778899"
        }else{
            if(it < 0) {isch.col[i] = "gray"}
            if(it > 0 & it < 60) {it = it + 1; isch.col[i] = "green"}
            if(it >= 60 & it < 240) {it = it + 1; isch.col[i] = "greenyellow"}
            if(it >= 240 & it < 360) {it = it + 1; isch.col[i] = "yellow"}
            if(it >= 360 & it < 900) {it = it + 1; isch.col[i] = "orange"}
            if(it >= 900) {it = it + 1; isch.col[i] = "red"}
        }   
    }
        
    # expression matrix of selected genes with changes
    tt = tissue.expr[sel.genes.ids,]
    row.genenames = gene2name[as.character(rownames(tt)),c("Description")]
    out_filename = paste("Heatmap.IschemicTimeIntervals_",tissue.name.nospace , ".pdf",sep="")
    
    if(mode == 1){
            # output heatmap.2
            pdf(out_filename,w=21,h=21)
                hcd = as.dendrogram(hclust(as.dist(1-cor(t(tt),method="pearson")), method="average"))
                heatmap.2(data.matrix(tt),col=bluered(150),scale="row",trace="none",dendrogram="none", key=TRUE, keysize=0.5, symkey=TRUE, cexRow=0.8,cexCol=0.8,notecex=1,ColSideColors=isch.col, Colv = FALSE, Rowv=hcd, labRow=row.genenames)
            dev.off()
    }else{
        # enrichment of selected genes
        gene.enrich = enrichedTermsGoKegg(tissue.name, sel.genes.ids, 0.05)
        # output heatmap.2
        rlab = c()
        rlab_names = c()
        for (i in 1:nrow(summary(gene.enrich$bp))){
            if(nrow(summary(gene.enrich$bp))>0){
                genes.col = gene2name[rownames(tt),]
                genes.to.mark = strsplit(summary(gene.enrich$bp)[i,"geneID"],"/")
                genes.col.lst = rep("gray",nrow(tt))
                genes.col.lst[which(genes.col$Description %in% unlist(genes.to.mark))] = "red"
                count_genes =  as.numeric(summary(gene.enrich$bp)[i,c("Count")])
                if(count_genes >= minNumGenes){
                    rlab = cbind(rlab, genes.col.lst)
                    rlab_names = c(rlab_names, summary(gene.enrich$bp)[i,c("Description")])
                }
            }
        }
        for (i in 1:nrow(summary(gene.enrich$mf))){
            if(nrow(summary(gene.enrich$mf))>0){
                genes.col = gene2name[rownames(tt),]
                genes.to.mark = strsplit(summary(gene.enrich$mf)[i,"geneID"],"/")
                genes.col.lst = rep("gray",nrow(tt))
                genes.col.lst[which(genes.col$Description %in% unlist(genes.to.mark))] = "red"
                count_genes =  as.numeric(summary(gene.enrich$mf)[i,c("Count")])
                if(count_genes >= minNumGenes){
                    rlab = cbind(rlab, genes.col.lst)
                    rlab_names = c(rlab_names, summary(gene.enrich$mf)[i,c("Description")])
                }
            }
        }
        for (i in 1:nrow(summary(gene.enrich$kegg))){
            if(nrow(summary(gene.enrich$kegg))>0){
                genes.col = gene2name[rownames(tt),]
                genes.to.mark = strsplit(summary(gene.enrich$kegg)[i,"geneID"],"/")
                genes.col.lst = rep("gray",nrow(tt))
                genes.col.lst[which(genes.col$Description %in% unlist(genes.to.mark))] = "red"
                count_genes =  as.numeric(summary(gene.enrich$kegg)[i,c("Count")])
                if(count_genes >= minNumGenes){
                    rlab = cbind(rlab, genes.col.lst)
                    rlab_names = c(rlab_names, summary(gene.enrich$kegg)[i,c("Description")])
                }
            }
        }
        # heatmap with multiple color side bars
        # Create color side bars
        colnames(rlab) = rlab_names
        pdf(out_filename,w=28,h=28)
        if(!is.null(ncol(rlab))){
            hcd = as.dendrogram(hclust(as.dist(1-cor(t(tt),method="pearson")), method="average"))
            heatmap.3(data.matrix(tt),col=bluered(150),scale="row",trace="none",dendrogram="none", key=TRUE, keysize=0.8, symkey=TRUE, cexRow=0.9,cexCol=0.8,notecex=1,ColSideColors=data.matrix(isch.col), Colv = FALSE, Rowv=hcd, labRow=gene2name[rownames(data.matrix(tt)),c("Description")], RowSideColors=data.matrix(t(rlab)),
            ColSideColorsSize=1, RowSideColorsSize=ncol(rlab),margins=c(6,12))
        }else{
            hcd = as.dendrogram(hclust(as.dist(1-cor(t(tt),method="pearson")), method="average"))
            heatmap.3(data.matrix(tt),col=bluered(150),scale="row",trace="none",dendrogram="none", key=TRUE, keysize=0.8, symkey=TRUE, cexRow=0.9,cexCol=0.8,notecex=1,ColSideColors=data.matrix(isch.col), Colv = FALSE, Rowv=hcd, labRow=gene2name[rownames(data.matrix(tt)),c("Description")], ColSideColorsSize=1, RowSideColorsSize=0,margins=c(6,12))
        }
        dev.off()
    }
}


######################################################################################
# The createHeatmapSelGenesIschemicTimeWrapper wraps several functions
# It call the function to determine genes with significant change between intervals; extend their info with additional information; creates heatmaps and performs enrichment analysis and stores the results;
# It receives the pval for significant change test and the number of genes to include in the heatmap
######################################################################################
createHeatmapSelGenesIschemicTimeWrapper <- function(pval, minNumGenes){
    for(tissue.name in sel.tissues.names[-c(1:30)]){
        sel.genes.description = c()
        cat (tissue.name, sep="\n")
        tissue.name.nospace = chartr(" ",".",tissue.name)
        out_filename = paste("Genes.WithSignifIntervalChange_",tissue.name.nospace, ".txt",sep="")

        # determines genes with significant change
        cat ("detect Time Interval Changes", sep="\n")
        sel.genes = detectTimeIntervalChanges(tissue.name, pval)
        
        # extend gene description
        sel.genes.description = merge(sel.genes, gene2name, by.x="gene", by.y="Name")
        sel.genes.description = merge(sel.genes.description, bt, by.x="gene",by.y="id")
        sel.genes.description =  cbind(sel.genes.description, rep(tissue.name,length(sel.genes$gene)))
        names(sel.genes.description)=c("Name", "Change","Description", "gene","Biotype","Tissue")
        
        # creates heatmap
        cat ("create heatmap", sep="\n")
        createHeatmapSelGenesIschemicTime(tissue.name, sel.genes$gene, minNumGenes, 1)
        write.table(sel.genes.description, out_filename,quote=F, row.names=F,sep="\t")
        
        # Enrichment analysis
        gene.enrich = enrichedTermsGoKegg(tissue.name, sel.genes.description, pval)       
        out_filename = paste("Genes.TimeChange.KEGG.",tissue.name.nospace, ".txt",sep="")
        write.table(summary(gene.enrich$kegg),out_filename,quote=F,row.names=F,sep="\t")
        out_filename = paste("Genes.TimeChange.MF.",tissue.name.nospace, ".txt",sep="")
        write.table(summary(gene.enrich$mf),out_filename,quote=F,row.names=F,sep="\t")
        out_filename = paste("Genes.TimeChange.BP.",tissue.name.nospace, ".txt",sep="")
        write.table(summary(gene.enrich$bp),out_filename,quote=F,row.names=F,sep="\t")
        out_filename = paste("Genes.TimeChange.CC.",tissue.name.nospace, ".txt",sep="")
        write.table(summary(gene.enrich$cc),out_filename,quote=F,row.names=F,sep="\t")   
    }
}


# Selection of gene lists
# in function createHeatmapSelGenesIschemicTimeWrapper the list of significan genes are stored in files names "Genes.WithSignifIntervalChange_*"
# in the command line we can further select genes of interest and create new lists. Example:
cat Genes.WithSignifIntervalChange_* | awk 'BEGIN{FS="\t"}{cnt[$4]=cnt[$4]+1;tis[$4]=tis[$4]";"$6;bt[$4]=$5;desc[$4]=desc[$4]";"$2;chg[$4]=chg[$4]";"$3}END{for(g in cnt){if(cnt[g]>=3) print g"\t"cnt[g]"\t"bt[g]"\t"desc[g]"\t"tis[g]"\t"chg[g]}}' | grep protein_coding | sort -nk2 > genes.TemporalDiffExp.ThreeOrMoreTissues.txt
cat Genes.WithSignifIntervalChange_* | grep protein_coding |  awk 'BEGIN{FS="\t"}{print $4"\t"$6"\t"$2}' | sort -k1 > pcod.gene.tissue.change.txt
cat Genes.WithSignifIntervalChange_* |  awk 'BEGIN{FS="\t"}{print $4"\t"$6"\t"$2}' | sort -k1 > allgenes.tissue.change.txt
